/***************************************************************************//**
* \file CAPSENSE_Structure.h
* \version 2.0
*
* \brief
*   This file provides the top level declarations of the component data structure.
*   Also, the file declares the high-level and low-level APIs for data access.
*
* \see CAPSENSE v2.0 Datasheet
*
*//*****************************************************************************
* Copyright (2016-2017), Cypress Semiconductor Corporation.
********************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress) and is
* protected by and subject to worldwide patent protection (United States and
* foreign), United States copyright laws and international treaty provisions.
* Cypress hereby grants to licensee a personal, non-exclusive, non-transferable
* license to copy, use, modify, create derivative works of, and compile the
* Cypress Source Code and derivative works for the sole purpose of creating
* custom software in support of licensee product to be used only in conjunction
* with a Cypress integrated circuit as specified in the applicable agreement.
* Any reproduction, modification, translation, compilation, or representation of
* this software except as specified above is prohibited without the express
* written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the
* materials described herein. Cypress does not assume any liability arising out
* of the application or use of any product or circuit described herein. Cypress
* does not authorize its products for use as critical components in life-support
* systems where a malfunction or failure may reasonably be expected to result in
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of
* such use and in doing so indemnifies Cypress against all charges. Use may be
* limited by and subject to the applicable Cypress software license agreement.
*******************************************************************************/

#if !defined(CY_SENSE_CAPSENSE_STRUCTURE_H)
#define CY_SENSE_CAPSENSE_STRUCTURE_H

#include "syslib/cy_syslib.h"
#include "cyfitter.h"
#include "CAPSENSE_Configuration.h"
#if (CAPSENSE_CSD_SS_DIS != CAPSENSE_CSD_AUTOTUNE)
    #include "CAPSENSE_SmartSense_LL.h"
#endif

#if (CAPSENSE_ENABLE == CAPSENSE_ALP_FILTER_EN)
    #include "CAPSENSE_AlpFilter_LL.h"
#endif

#if ((CAPSENSE_ENABLE == CAPSENSE_GES_GLOBAL_EN) ||\
     (CAPSENSE_ENABLE == CAPSENSE_BALLISTIC_MULTIPLIER_EN))
    #include "CAPSENSE_TMG.h"
#endif

#if (CAPSENSE_ENABLE == CAPSENSE_CENTROID_5X5_CSD_EN)
    #include "CAPSENSE_AdvancedCentroid_LL.h"
#endif

#if (CAPSENSE_ENABLE == CAPSENSE_POS_ADAPTIVE_IIR_FILTER_EN)
    #include "CAPSENSE_AdaptiveFilter_LL.h"
#endif

/*******************************************************************************
* Constant Definitions
*******************************************************************************/

/* Defines size of Sensor Status Register in Data Structure */
#define CAPSENSE_SNS_STS_TYPE               uint8

/* No touch condition for slider position report */
#define CAPSENSE_SLIDER_NO_TOUCH            (0xFFFFu)
#define CAPSENSE_TOUCHPAD_NO_TOUCH          (0xFFFFFFFFLu)

#define CAPSENSE_SLIDER_POS_NONE            (0xFFFFu)
#define CAPSENSE_TOUCHPAD_POS_NONE          (0xFFFFu)

#define CAPSENSE_MATRIX_POS_NONE            (0xFFu)
#define CAPSENSE_MATRIX_POS_MULTI           (0xFEu)

#define CAPSENSE_PROX_STS_MASK              (3Lu)
#define CAPSENSE_PROX_STS_OFFSET(proxId)    ((proxId) << 1u)

#define CAPSENSE_MATRIX_BUTTONS_TOUCHED     (0x80000000Lu)

/*******************************************************************************
* Enumeration types definition
*******************************************************************************/

/***************************************************************************//**
* \brief Defines widget types
*******************************************************************************/
typedef enum
{
    CAPSENSE_WD_BUTTON_E        = 0x01u,
    CAPSENSE_WD_LINEAR_SLIDER_E = 0x02u,
    CAPSENSE_WD_RADIAL_SLIDER_E = 0x03u,
    CAPSENSE_WD_MATRIX_BUTTON_E = 0x04u,
    CAPSENSE_WD_TOUCHPAD_E      = 0x05u,
    CAPSENSE_WD_PROXIMITY_E     = 0x06u
} CAPSENSE_WD_TYPE_ENUM;


/***************************************************************************//**
* \brief Defines sensing methods types
*******************************************************************************/
typedef enum
{
    CAPSENSE_UNDEFINED_E            = 0x00u,
    CAPSENSE_SENSE_METHOD_CSD_E     = 0x01u,
    CAPSENSE_SENSE_METHOD_CSX_E     = 0x02u,
    CAPSENSE_SENSE_METHOD_BIST_E    = 0x03u
} CAPSENSE_SENSE_METHOD_ENUM;

/***************************************************************************//**
* \brief Defines electrode types
*******************************************************************************/
typedef enum
{
    CAPSENSE_ELTD_TYPE_SELF_E   = 0x01u,
    CAPSENSE_ELTD_TYPE_MUT_TX_E = 0x02u,
    CAPSENSE_ELTD_TYPE_MUT_RX_E = 0x03u
} CAPSENSE_ELTD_TYPE_ENUM;

/**
* \cond SECTION_STRUCTURES
* \addtogroup group_structures
* \{
*/
/*******************************************************************************
* Declares RAM structures for all used widgets
*******************************************************************************/

/***************************************************************************//**
* \brief Declares common widget RAM parameters
*******************************************************************************/
typedef struct
{
    /**
     *  Provides scan resolution for the CSD Widgets.
     *  Provides number of the sub-conversions for the CSX Widgets.
     */
    uint16 resolution;

    /**
     *  Widget Finger Threshold.
     */
    CAPSENSE_THRESHOLD_TYPE fingerTh;

    /**
     *  Widget Noise Threshold.
     */
    uint8  noiseTh;

    /**
     *  Widget Negative Noise Threshold.
     */
    uint8  nNoiseTh;

    /**
     *  Widget Hysteresis for the signal crossing finger threshold.
     */
    uint8  hysteresis;

    /**
     *  Widget Debounce for the signal above the finger threshold. OFF to ON.
     */
    uint8  onDebounce;

    /**
     *  The widget low baseline reset count.
     *  Specifies the number of samples the sensor signal must be below the Negative 
     *  Noise Threshold to trigger a baseline reset.
     */
    CAPSENSE_LOW_BSLN_RST_TYPE lowBslnRst;

    /**
     *  Sets the current of the modulation IDAC for the CSD widgets. 
     *  For the CSD Touchpad and Matrix Button widgets sets the current of the 
     *  modulation IDAC for the column sensors.
     *  Not used for the CSX widgets.
     */
    uint8  idacMod [CAPSENSE_NUM_SCAN_FREQS];

    /**
     *  Specifies the sense clock divider. Present only if individual clock 
     *  dividers are enabled.
     *  Specifies the sense clock divider for the Column sensors for the Matrix Buttons
     *  and Touchpad widgets.
     *  Sets Tx clock divider for the CSX Widgets.
     */
    uint16 snsClk;

    /**
     *  Register for internal use
     */
    uint8  snsClkSource;

    /**
     *  Widget Finger capacitance parameter. Available only if the 
     *  SmartSense is enabled.
     *  Not used for the CSX Widgets.
     */
    uint16 fingerCap;

    /**
     *  The 75% of signal per user-defined finger capacitance
     */
    uint16 sigPFC;
} CAPSENSE_RAM_WD_BASE_STRUCT;

/***************************************************************************//**
* \brief Declares RAM parameters for the Slider
*******************************************************************************/
typedef struct
{
    /**
     *  Provides scan resolution for the CSD Widgets.
     *  Provides number of the sub-conversions for the CSX Widgets.
     */
    uint16 resolution;

    /**
     *  Widget Finger Threshold.
     */
    CAPSENSE_THRESHOLD_TYPE fingerTh;

    /**
     *  Widget Noise Threshold.
     */
    uint8  noiseTh;

    /**
     *  Widget Negative Noise Threshold.
     */
    uint8  nNoiseTh;

    /**
     *  Widget Hysteresis for the signal crossing finger threshold.
     */
    uint8  hysteresis;

    /**
     *  Widget Debounce for the signal above the finger threshold. OFF to ON.
     */
    uint8  onDebounce;

    /**
     *  The widget low baseline reset count.
     *  Specifies the number of samples the sensor signal must be below the Negative 
     *  Noise Threshold to trigger a baseline reset.
     */
    CAPSENSE_LOW_BSLN_RST_TYPE lowBslnRst;

    /**
     *  Sets the current of the modulation IDAC for the CSD widgets. 
     *  For the CSD Touchpad and Matrix Button widgets sets the current of the 
     *  modulation IDAC for the column sensors.
     *  Not used for the CSX widgets.
     */
    uint8  idacMod [CAPSENSE_NUM_SCAN_FREQS];

    /**
     *  Specifies the sense clock divider. Present only if individual clock 
     *  dividers are enabled.
     *  Specifies the sense clock divider for the Column sensors for the Matrix Buttons
     *  and Touchpad widgets.
     *  Sets Tx clock divider for the CSX Widgets.
     */
    uint16 snsClk;

    /**
     *  Register for internal use
     */
    uint8  snsClkSource;

    /**
     *  Widget Finger capacitance parameter. Available only if the 
     *  SmartSense is enabled.
     *  Not used for the CSX Widgets.
     */
    uint16 fingerCap;

    /**
     *  The 75% of signal per user-defined finger capacitance
     */
    uint16 sigPFC;

    /**
     *  Reports the widget position.
     */
    uint16 position [CAPSENSE_NUM_CENTROIDS];
} CAPSENSE_RAM_WD_SLIDER_STRUCT;


/***************************************************************************//**
* \brief Declares RAM structure with all defined widgets
*******************************************************************************/
typedef struct
{
    /**
     *  LinearSlider widget RAM structure
     */
    CAPSENSE_RAM_WD_SLIDER_STRUCT linearslider;
} CAPSENSE_RAM_WD_LIST_STRUCT;


/***************************************************************************//**
* \brief Declares RAM structure for sensors
*******************************************************************************/
typedef struct
{
    /**
     *  The sensor raw counts.
     */
    uint16 raw [CAPSENSE_NUM_SCAN_FREQS];

    /**
     *  The sensor baseline.
     */
    uint16 bsln [CAPSENSE_NUM_SCAN_FREQS];

    /**
     *  For the bucket baseline algorithm holds the bucket state,
     *  For the IIR baseline keeps LSB of the baseline value.
     */
    uint8  bslnExt [CAPSENSE_NUM_SCAN_FREQS];

    /**
     *  Sensor differences.
     */
    CAPSENSE_THRESHOLD_TYPE diff;

    /**
     *  The baseline reset counter for the low baseline reset function.
     */
    CAPSENSE_LOW_BSLN_RST_TYPE negBslnRstCnt [CAPSENSE_NUM_SCAN_FREQS];

    /**
     *  CSD Widgets: The compensation IDAC value.
     *  CSX Widgets: The balancing IDAC value.
     */
    uint8  idacComp [CAPSENSE_NUM_SCAN_FREQS];
} CAPSENSE_RAM_SNS_STRUCT;


/***************************************************************************//**
* \brief Declares RAM structure with all defined sensors
*******************************************************************************/
typedef struct
{
    /**
     *  LinearSlider sensors RAM structures array
     */
    CAPSENSE_RAM_SNS_STRUCT linearslider [CAPSENSE_LINEARSLIDER_NUM_SENSORS];
} CAPSENSE_RAM_SNS_LIST_STRUCT;


/***************************************************************************//**
* \brief Declares the top-level RAM Data Structure
*******************************************************************************/
typedef struct
{
    /**
     *  16-bit CRC calculated by the customizer for the component 
     *  configuration. 
     *  Used by the Tuner application to identify if the FW corresponds to the 
     *  specific user configuration.
     */
    uint16 configId;

    /**
     *  Used by the Tuner application to identify device-specific configuration.
     */
    uint16 deviceId;

    /**
     *  Used by the Tuner application to identify the system clock frequency.
     */
    uint16 hwClock;

    /**
     *  Tuner Command Register. 
     *  Used for the communication between the Tuner GUI and the component.
     */
    uint16 tunerCmd;

    /**
     *  This counter gets incremented after each scan.
     */
    uint16 scanCounter;

    /**
     *  Status information: Current Widget, Scan active, Error code.
     */
    volatile uint32 status;

    /**
     *  The bitmask that sets which Widgets are enabled and scanned, 
     *  each bit corresponds to one widget.
     */
    uint32 wdgtEnable [CAPSENSE_WDGT_STATUS_WORDS];

    /**
     *  The bitmask that reports activated Widgets (widgets that detect 
     *  a touch signal above the threshold), each bit corresponds to one widget.
     */
    uint32 wdgtStatus [CAPSENSE_WDGT_STATUS_WORDS];

    /**
     *  For Buttons, Sliders, Matrix Buttons and CSD Touchpad each bit 
     *  reports status of the individual sensor of the widget:
     *  1 - active (above the finger threshold);
     *  0 - inactive;
     *  For the CSD Touchpad and CSD Matrix Buttons, the column sensors occupy the 
     *  least significant bits.
     *  For the Proximity widget, each sensor uses two bits with the following meaning:
     *  00 - Not active;
     *  01 - Proximity detected (signal above finger threshold);
     *  11 - A finger touch detected (signal above the touch threshold);
     *  For the CSX Touchpad Widget, this register provides a number of the detected 
     *  touches. The array size is equal to the total number of widgets. The size of the array 
     *  element depends on the max number of sensors per widget used in the current design. 
     *  It could be 1, 2 or 4 bytes.
     */
    CAPSENSE_SNS_STS_TYPE snsStatus [CAPSENSE_TOTAL_WIDGETS];

    /**
     *  The configuration register for global parameters of the CSD0 block.
     */
    uint16 csd0Config;

    /**
     *  The modulator clock divider for the CSD widgets.
     */
    uint8  modCsdClk;

    /**
     *  RAM Widget Objects.
     */
    CAPSENSE_RAM_WD_LIST_STRUCT wdgtList;

    /**
     *  RAM Sensor Objects.
     */
    CAPSENSE_RAM_SNS_LIST_STRUCT snsList;

    /**
     *  The selected widget ID.
     */
    uint8  snrTestWidgetId;

    /**
     *  The selected sensor ID.
     */
    uint8  snrTestSensorId;

    /**
     *  The scan counter.
     */
    uint16 snrTestScanCounter;

    /**
     *  The sensor raw counts.
     */
    uint16 snrTestRawCount [CAPSENSE_NUM_SCAN_FREQS];
} CAPSENSE_RAM_STRUCT;


/***************************************************************************//**
* \brief Declares the Flash IO object
*******************************************************************************/
typedef struct
{
    /**
     *  Pointer to the base port register of the IO.
     */
    GPIO_PRT_Type* pcPtr;

    /**
     *  Position of the IO in the port.
     */
    uint8    pinNumber;
} CAPSENSE_FLASH_IO_STRUCT;


/***************************************************************************//**
* \brief Declares the Flash Electrode object
*******************************************************************************/
typedef struct
{
    /**
     *  Index of the first IO in the Flash IO Object Array.
     */
    uint16 firstPinId;

    /**
     *  Total number of IOs in this sensor.
     */
    uint8  numPins;

    /**
     * Sensor type:
     * - ELTD_TYPE_SELF_E - CSD sensor;
     * - ELTD_TYPE_MUT_TX_E - CSX Tx sensor;
     * - ELTD_TYPE_MUT_RX_E - CSX Rx sensor;
     */
    uint8  type;
} CAPSENSE_FLASH_SNS_STRUCT;


/***************************************************************************//**
* \brief Declares the structure with all Flash electrode objects
*******************************************************************************/
typedef struct
{
    /**
     *  No ganged sensors available
     */
    uint8 notUsed;
} CAPSENSE_FLASH_SNS_LIST_STRUCT;




/***************************************************************************//**
* \brief Declares Flash widget object
*******************************************************************************/
typedef struct
{
    /**
     *  Points to the array of the FLASH Sensor Objects or FLASH IO Objects that belong to this widget.
     *  Sensing block uses this pointer to access and configure IOs for the scanning.
     *  Bit #2 in WD_STATIC_CONFIG field indicates the type of array: 
     *  1 - Sensor Object; 
     *  0 - IO Object.
     */
    void const * ptr2SnsFlash;

    /**
     *  Points to the Widget Object in RAM. Sensing block uses it to 
     *  access scan parameters.
     *  Processing uses it to access threshold and widget specific data.
     */
    void * ptr2WdgtRam;

    /**
     *  Points to the array of Sensor Objects in RAM.
     *  The sensing and processing blocks use it to access the scan data.
     */
    CAPSENSE_RAM_SNS_STRUCT * ptr2SnsRam;

    /**
     *  Points to the array of the Filter History Objects in RAM
     */
    void * ptr2FltrHistory;

    /**
     *  Points to the array of the debounce counters.
     *  The size of the debounce counter is 8 bits.
     *  These arrays are not part of the data structure.
     */
    uint8 * ptr2DebounceArr;

    /**
     *  Miscellaneous configuration flags.
     */
    uint32 staticConfig;

    /**
     *  The total number of sensors. 
     *  For CSD widgets: WD_NUM_ROWS + WD_NUM_COLS. 
     *  For CSX widgets: WD_NUM_ROWS * WD_NUM_COLS.
     */
    uint16 totalNumSns;

    /**
     *  Specifies one of the following widget types: 
     *  WD_BUTTON_E,
     *  WD_LINEAR_SLIDER_E,
     *  WD_RADIAL_SLIDER_E,
     *  WD_MATRIX_BUTTON_E, 
     *  WD_TOUCHPAD_E, 
     *  WD_PROXIMITY_E
     */
    uint8  wdgtType;

    /**
     *  For CSD Button and Proximity Widgets, the number of sensors.
     *  For CSD Slider Widget, the number of segments.
     *  For CSD Touchpad and Matrix Button, the number of the column sensors.
     *  For CSX Button, Touchpad and Matrix Button, the number of the Rx electrodes.
     */
    uint8  numCols;

    /**
     *  Sliders: The Linear/Angular resolution.
     *  Touchpad: The X-Axis resolution.
     */
    uint16 xResolution;

    /**
     *  The pre-calculated X resolution centroid multiplier used for 
     *  the X-axis position calculation.
     *  Calculated as follows:
     *  RADIAL: (WD_X_RESOLUTION * 256) / WD_NUM_COLS;
     *  LINEAR and TOUCHPAD: (WD_X_RESOLUTION * 256) / (WD_NUM_COLS - CONFIG);
     *  where CONFIG is 0 or 1 depends on CentroidMultiplerMethod parameter
     */
    uint32 xCentroidMultiplier;

    /**
     *  The pointer to the array with the sensor noise envelope data.
     *  Set to the valid value only for the CSD widgets.
     *  For the CSX widgets, this pointer is set to NULL.
     *  The pointed array is not part of the data structure.
     */
    SMARTSENSE_CSD_NOISE_ENVELOPE_STRUCT * ptr2NoiseEnvlp;

    /**
     *  The position IIR filter coefficient.
     */
    uint8  iirFilterCoeff;
} CAPSENSE_FLASH_WD_STRUCT;


/***************************************************************************//**
* \brief Declares top-level Flash Data Structure
*******************************************************************************/
typedef struct
{
    /**
     *  Array of flash widget objects
     */
    CAPSENSE_FLASH_WD_STRUCT wdgtArray[CAPSENSE_TOTAL_WIDGETS];
} CAPSENSE_FLASH_STRUCT;


#if (CAPSENSE_ENABLE == CAPSENSE_TST_BSLN_RAW_OUT_RANGE_EN)
    /***************************************************************************//**
    * \brief Defines the structure for test of baseline and raw count limits which
    * will be determined by user for every sensor grounding on the
    * manufacturing specific data
    *******************************************************************************/
    typedef struct
    {
        /**
         *  Upper limit of a sensor baseline.
         */
        uint16 bslnHiLim;
        /**
         *  Lower limit of a sensor baseline.
         */
        uint16 bslnLoLim;
        /**
         *  Upper limit of a sensor raw count.
         */
        uint16 rawHiLim;
        /**
         *  Lower limit of a sensor raw count.
         */
        uint16 rawLoLim;
    } CAPSENSE_BSLN_RAW_RANGE_STRUCT;
#endif /* (CAPSENSE_ENABLE == CAPSENSE_TST_BSLN_RAW_OUT_RANGE_EN) */

/** \}
* \endcond */


/***************************************************************************//**
* Declares Dual-channel scan structure
*******************************************************************************/
typedef struct
{
    uint8 csd0WidgetId;
    uint8 csd0SensorId;
    uint8 csd1WidgetId;
    uint8 csd1SensorId;
} CAPSENSE_SCAN_SLOT_STRUCT;


/***************************************************************************//**
* Declares Filter module structures
*******************************************************************************/

/***************************************************************************//**
* \brief Declares filter channel structure for regular sensors
*******************************************************************************/
typedef struct
{
    uint16 notUsed;
} CAPSENSE_REGULAR_FLTR_CHANNEL_STRUCT;

/***************************************************************************//**
* \brief Declares filter structure for regular sensors
*******************************************************************************/
typedef struct
{
    /**
     *  Array of CAPSENSE_REGULAR_FLTR_CHANNEL_STRUCT for each available scan frequency
     */
    CAPSENSE_REGULAR_FLTR_CHANNEL_STRUCT regularChannel[CAPSENSE_NUM_SCAN_FREQS];
} CAPSENSE_REGULAR_FLTR_STRUCT;

/***************************************************************************//**
* \brief Declares union for filter structure variants
*******************************************************************************/
typedef union
{
    /**
     *  Pointer to void type
     */
    void *ptr;

    /**
     *  Pointer to CAPSENSE_REGULAR_FLTR_STRUCT type
     */
    CAPSENSE_REGULAR_FLTR_STRUCT *ptrRegular;
} CAPSENSE_PTR_FILTER_VARIANT;


#if (0u != CAPSENSE_POSITION_FILTER_EN)
typedef struct
{
    #if (0u != CAPSENSE_POS_MEDIAN_FILTER_EN)
        uint16 posMedianZ1;
        uint16 posMedianZ2;
    #endif /* #if (0u != CAPSENSE_POS_MEDIAN_FILTER_EN) */

    #if (0u != CAPSENSE_POS_IIR_FILTER_EN)
        uint16 posIIR;
    #endif /* #if (0u != CAPSENSE_POS_IIR_FILTER_EN) */

    #if (0u != CAPSENSE_POS_ADAPTIVE_IIR_FILTER_EN)
        uint16 posAIIR;
    #endif /* (0u != CAPSENSE_POS_ADAPTIVE_IIR_FILTER_EN) */

    #if (0u != CAPSENSE_POS_AVERAGE_FILTER_EN)
        uint16 posAverage;
    #endif /* #if (0u != CAPSENSE_POS_AVERAGE_FILTER_EN) */

    #if (0u != CAPSENSE_POS_JITTER_FILTER_EN)
        uint16 posJitter;
    #endif /* #if (0u != CAPSENSE_POS_JITTER_FILTER_EN) */

    #if (0u != CAPSENSE_POS_ADAPTIVE_IIR_FILTER_EN)
        uint8 posAIIRCoeff;
    #endif /* (0u != CAPSENSE_POS_ADAPTIVE_IIR_FILTER_EN) */
} CAPSENSE_SLIDER_POS_HISTORY_STRUCT;

typedef struct
{
    CAPSENSE_SLIDER_POS_HISTORY_STRUCT xPos;
    CAPSENSE_SLIDER_POS_HISTORY_STRUCT yPos;
} CAPSENSE_TOUCHPAD_POS_HISTORY_STRUCT;
#endif /* (0u != CAPSENSE_POSITION_FILTER_EN) */

/*******************************************************************************
* API Constants
*******************************************************************************/

/*******************************************************************************
* Defines shift/mask values for data structure register bit fields
*******************************************************************************/

/* CSD0_CONFIG bit fields */

/* Bit 0: The multi-frequency scan enable/disable at runtime. This bit is functional
 * only if the multi-frequency scan functionality has been activated
 * in general component configuration. */
#define CAPSENSE_MULTI_FREQ_EN_SIZE       (0x00000001Lu)
#define CAPSENSE_MULTI_FREQ_EN_SHIFT      (0u)
#define CAPSENSE_MULTI_FREQ_EN_MASK       (CAPSENSE_MULTI_FREQ_EN_SIZE << CAPSENSE_MULTI_FREQ_EN_SHIFT)

/* Bit 1: The sensor auto-reset is enabled */
#define CAPSENSE_SNS_ARST_SIZE            (0x00000001Lu)
#define CAPSENSE_SNS_ARST_SHIFT           (1u)
#define CAPSENSE_SNS_ARST_MASK            (CAPSENSE_SNS_ARST_SIZE << CAPSENSE_SNS_ARST_SHIFT)

/* Bits 2-3: The IDAC range setting (4x/8x) */
#define CAPSENSE_IDAC_RANGE_SIZE          (0x00000003Lu)
#define CAPSENSE_IDAC_RANGE_SHIFT         (2u)
#define CAPSENSE_IDAC_RANGE_MASK          (CAPSENSE_IDAC_RANGE_SIZE << CAPSENSE_IDAC_RANGE_SHIFT)

/* Bits 4-7: Reserved */

/* Bit 8: The shield electrode signal Enable/Disable */
#define CAPSENSE_SHLD_EN_SIZE             (0x00000001Lu)
#define CAPSENSE_SHLD_EN_SHIFT            (8u)
#define CAPSENSE_SHLD_EN_MASK             (CAPSENSE_SHLD_EN_SIZE << CAPSENSE_SHLD_EN_SHIFT)

/* Bits 9-10: Selects the delay by which the csd_shield is delayed relative to csd_sense */
#define CAPSENSE_SHLD_DLY_SIZE            (0x00000003Lu)
#define CAPSENSE_SHLD_DLY_SHIFT           (9u)
#define CAPSENSE_SHLD_DLY_MASK            (CAPSENSE_SHLD_DLY_SIZE << CAPSENSE_SHLD_DLY_SHIFT)


/* STATUS bit fields */

/* Bits 0-6: The widget currently selected for CSD0 */
#define CAPSENSE_STATUS_WDGT0_SIZE        (0x0000007FLu)
#define CAPSENSE_STATUS_WDGT0_SHIFT       (0u)
#define CAPSENSE_STATUS_WDGT0_MASK        (CAPSENSE_STATUS_WDGT0_SIZE << CAPSENSE_STATUS_WDGT0_SHIFT)

/* Bit 7: The CSD0 block status */
#define CAPSENSE_STATUS_CSD0_SIZE         (0x00000001Lu)
#define CAPSENSE_STATUS_CSD0_SHIFT        (7u)
#define CAPSENSE_STATUS_CSD0_MASK         (CAPSENSE_STATUS_CSD0_SIZE << CAPSENSE_STATUS_CSD0_SHIFT)

/* Bits 8-14: The widget currently selected for CSD1 */
#define CAPSENSE_STATUS_WDGT1_SIZE        (0x0000007FLu)
#define CAPSENSE_STATUS_WDGT1_SHIFT       (8u)
#define CAPSENSE_STATUS_WDGT1_MASK        (CAPSENSE_STATUS_WDGT1_SIZE << CAPSENSE_STATUS_WDGT1_SHIFT)

/* Bit 15: The CSD1 block status */
#define CAPSENSE_STATUS_CSD1_SIZE         (0x00000001Lu)
#define CAPSENSE_STATUS_CSD1_SHIFT        (15u)
#define CAPSENSE_STATUS_CSD1_MASK         (CAPSENSE_STATUS_CSD1_SIZE << CAPSENSE_STATUS_CSD1_SHIFT)

/* Bit 16: The WIDGET BUSY flag status for CSD0 */
#define CAPSENSE_STATUS_WDGT0_BUSY_SIZE   (0x00000001Lu)
#define CAPSENSE_STATUS_WDGT0_BUSY_SHIFT  (16u)
#define CAPSENSE_STATUS_WDGT0_BUSY_MASK   (CAPSENSE_STATUS_WDGT0_BUSY_SIZE << CAPSENSE_STATUS_WDGT0_BUSY_SHIFT)

/* Bit 17: The WIDGET BUSY flag status for CSD1 */
#define CAPSENSE_STATUS_WDGT1_BUSY_SIZE   (0x00000001Lu)
#define CAPSENSE_STATUS_WDGT1_BUSY_SHIFT  (17u)
#define CAPSENSE_STATUS_WDGT1_BUSY_MASK   (CAPSENSE_STATUS_WDGT1_BUSY_SIZE << CAPSENSE_STATUS_WDGT1_BUSY_SHIFT)

/* Bits 18-23: Reserved */

/* Bits 24-31: CapSense component error code */
#define CAPSENSE_STATUS_ERR_SIZE          (0x000000FFLu)
#define CAPSENSE_STATUS_ERR_SHIFT         (24u)
#define CAPSENSE_STATUS_ERR_MASK          (CAPSENSE_STATUS_ERR_SIZE << CAPSENSE_STATUS_ERR_SHIFT)


/* WD_STATIC_CONFIG bit fields */

/* Bit 0: The sense/Tx frequency selection: 0 - Common, 1 - Individual (per widget). */
#define CAPSENSE_SNS_FREQ_SIZE            (0x00000001Lu)
#define CAPSENSE_SNS_FREQ_SHIFT           (0u)
#define CAPSENSE_SNS_FREQ_MASK            (CAPSENSE_SNS_FREQ_SIZE << CAPSENSE_SNS_FREQ_SHIFT)

/* Bit 1: Duplexing Enable/Disable for linear sliders. */
#define CAPSENSE_DIPLEXING_SIZE           (0x00000001Lu)
#define CAPSENSE_DIPLEXING_SHIFT          (1u)
#define CAPSENSE_DIPLEXING_MASK           (CAPSENSE_DIPLEXING_SIZE << CAPSENSE_DIPLEXING_SHIFT)

/* Bit 2: The widget contains a sensor ganged to other sensors. */
#define CAPSENSE_GANGED_SNS_SIZE          (0x00000001Lu)
#define CAPSENSE_GANGED_SNS_SHIFT         (2u)
#define CAPSENSE_GANGED_SNS_MASK          (CAPSENSE_GANGED_SNS_SIZE << CAPSENSE_GANGED_SNS_SHIFT)

/* Bit 3: Some pin from this widget is used by other sensors to make a ganged sensor. */
#define CAPSENSE_SHARED_IO_SIZE           (0x00000001Lu)
#define CAPSENSE_SHARED_IO_SHIFT          (3u)
#define CAPSENSE_SHARED_IO_MASK           (CAPSENSE_SHARED_IO_SIZE << CAPSENSE_SHARED_IO_SHIFT)

/* Bit 4: The centroid position IIR filter Enable/Disable. */
#define CAPSENSE_POS_IIR_FILTER_SIZE      (0x00000001Lu)
#define CAPSENSE_POS_IIR_FILTER_SHIFT     (4u)
#define CAPSENSE_POS_IIR_FILTER_MASK      (CAPSENSE_POS_IIR_FILTER_SIZE << CAPSENSE_POS_IIR_FILTER_SHIFT)

/* Bit 5: The centroid position median filter Enable/Disable. */
#define CAPSENSE_POS_MEDIAN_FILTER_SIZE   (0x00000001Lu)
#define CAPSENSE_POS_MEDIAN_FILTER_SHIFT  (5u)
#define CAPSENSE_POS_MEDIAN_FILTER_MASK   (CAPSENSE_POS_MEDIAN_FILTER_SIZE << CAPSENSE_POS_MEDIAN_FILTER_SHIFT)

/* Bit 6: The centroid position average filter Enable/Disable. */
#define CAPSENSE_POS_AVERAGE_FILTER_SIZE  (0x00000001Lu)
#define CAPSENSE_POS_AVERAGE_FILTER_SHIFT (6u)
#define CAPSENSE_POS_AVERAGE_FILTER_MASK  (CAPSENSE_POS_AVERAGE_FILTER_SIZE << CAPSENSE_POS_AVERAGE_FILTER_SHIFT)

/* Bit 7: The centroid position jitter filter Enable/Disable. */
#define CAPSENSE_POS_JITTER_FILTER_SIZE   (0x00000001Lu)
#define CAPSENSE_POS_JITTER_FILTER_SHIFT  (7u)
#define CAPSENSE_POS_JITTER_FILTER_MASK   (CAPSENSE_POS_JITTER_FILTER_SIZE << CAPSENSE_POS_JITTER_FILTER_SHIFT)

/* Bit 8: The multiphase Tx scan Enable (CSX widgets only). */
#define CAPSENSE_MULTIPHASE_TX_SIZE       (0x00000001Lu)
#define CAPSENSE_MULTIPHASE_TX_SHIFT      (8u)
#define CAPSENSE_MULTIPHASE_TX_MASK       (CAPSENSE_MULTIPHASE_TX_SIZE << CAPSENSE_MULTIPHASE_TX_SHIFT)

/* Bit 9: The centroid position adaptive IIR filter Enable/Disable. */
#define CAPSENSE_AIIR_FILTER_SIZE         (0x00000001Lu)
#define CAPSENSE_AIIR_FILTER_SHIFT        (9u)
#define CAPSENSE_AIIR_FILTER_MASK         (CAPSENSE_AIIR_FILTER_SIZE << CAPSENSE_AIIR_FILTER_SHIFT)

/* Bit 10: Ballistic multiplier Enable/Disable. */
#define CAPSENSE_BALLISTIC_SIZE           (0x00000001Lu)
#define CAPSENSE_BALLISTIC_SHIFT          (10u)
#define CAPSENSE_BALLISTIC_MASK           (CAPSENSE_BALLISTIC_SIZE << CAPSENSE_BALLISTIC_SHIFT)

/* Bit 11: 3x3 centroid Enable/Disable. */
#define CAPSENSE_CENTROID_3X3_SIZE        (0x00000001Lu)
#define CAPSENSE_CENTROID_3X3_SHIFT       (11u)
#define CAPSENSE_CENTROID_3X3_MASK        (CAPSENSE_CENTROID_3X3_SIZE << CAPSENSE_CENTROID_3X3_SHIFT)

/* Bit 12: 5x5 centroid Enable/Disable. */
#define CAPSENSE_CENTROID_5X5_SIZE        (0x00000001Lu)
#define CAPSENSE_CENTROID_5X5_SHIFT       (12u)
#define CAPSENSE_CENTROID_5X5_MASK        (CAPSENSE_CENTROID_5X5_SIZE << CAPSENSE_CENTROID_5X5_SHIFT)

/* Bit 13: Edge correction Enable/Disable. */
#define CAPSENSE_EDGE_CORRECTION_SIZE     (0x00000001Lu)
#define CAPSENSE_EDGE_CORRECTION_SHIFT    (13u)
#define CAPSENSE_EDGE_CORRECTION_MASK     (CAPSENSE_EDGE_CORRECTION_SIZE << CAPSENSE_EDGE_CORRECTION_SHIFT)

/* Bit 14: Two finger detection Enable/Disable. */
#define CAPSENSE_TWO_FINGER_DETECTION_SIZE (0x00000001Lu)
#define CAPSENSE_TWO_FINGER_DETECTION_SHIFT (14u)
#define CAPSENSE_TWO_FINGER_DETECTION_MASK (CAPSENSE_TWO_FINGER_DETECTION_SIZE << CAPSENSE_TWO_FINGER_DETECTION_SHIFT)


/*******************************************************************************
* Defines Data Structure Macro helpers
*******************************************************************************/

/*******************************************************************************
* Determines the widget specific word in the wdgtStatus array by widget ID
*******************************************************************************/
/* Divide by 32 - size of the wdgtStatus word in bits to find the word index */
#define CAPSENSE_GET_WDGT_STATUS_INDEX(wdgtId)  ((wdgtId) >> 5u)

/*******************************************************************************
* Determines the widget bitmask in wdgtStatus word by widget ID
*******************************************************************************/
/* Take the least 5 bits of widget id to find the bit number */
#define CAPSENSE_GET_WDGT_STATUS_MASK(wdgtId)   (1Lu << ((wdgtId) & 0x1FLu))

/*******************************************************************************
* Determines the sense method of the widget
*******************************************************************************/
#if (0u != CAPSENSE_TOTAL_CSD_WIDGETS) && (0u != CAPSENSE_TOTAL_CSX_WIDGETS)
    #define CAPSENSE_GET_SENSE_METHOD(ptrFlash) ((CAPSENSE_SENSE_METHOD_ENUM)(ptrFlash)->senseMethod)
#else
    #if (0u != CAPSENSE_TOTAL_CSX_WIDGETS)
        #define CAPSENSE_GET_SENSE_METHOD(ptrFlash) CAPSENSE_SENSE_METHOD_CSX_E
    #else
        #define CAPSENSE_GET_SENSE_METHOD(ptrFlash) CAPSENSE_SENSE_METHOD_CSD_E
    #endif /* #if (0u != CAPSENSE_TOTAL_CSX_WIDGETS) */
#endif /* #if (0u != CAPSENSE_TOTAL_CSD_WIDGETS) && (0u != CAPSENSE_TOTAL_CSX_WIDGETS) */

/*******************************************************************************
* Gets the widget type
*******************************************************************************/
#define CAPSENSE_GET_WIDGET_TYPE(ptrFlashWdgt)  ((CAPSENSE_WD_TYPE_ENUM)((ptrFlashWdgt)->wdgtType))

/*******************************************************************************
* Gets the number of the widget's sensors
*******************************************************************************/
#define CAPSENSE_GET_SENSOR_COUNT(widgetId)         CAPSENSE_dsFlash.wdgtArray[(widgetId)].totalNumSns
#define CAPSENSE_GET_SNS_CNT_BY_PTR(ptrFlashWidget) (ptrFlashWidget)->totalNumSns

/*******************************************************************************
* Increments the pointer to the Regular Filter History Object
*******************************************************************************/
#if (0u != CAPSENSE_REGULAR_RC_ALP_FILTER_EN)
    #define CAPSENSE_INC_REG_FLTR_OBJ(fltrVariant)      \
        do {                                                    \
            (fltrVariant).ptrAlp++;                             \
        } while(0)
#elif (0u != CAPSENSE_REGULAR_RC_FILTER_EN)
    #define CAPSENSE_INC_REG_FLTR_OBJ(fltrVariant)      \
        do {                                                    \
            (fltrVariant).ptrRegular++;                         \
        } while(0)
#else
    #define CAPSENSE_INC_REG_FLTR_OBJ(fltrVariant)      \
        do {                                                    \
        } while(0)
#endif

/*******************************************************************************
* Increments the pointer to the Proximity Filter History Object
*******************************************************************************/
#if (0u != CAPSENSE_PROX_RC_ALP_FILTER_EN)
    #define CAPSENSE_INC_PROX_FLTR_OBJ(fltrVariant)     \
        do {                                                    \
            (fltrVariant).ptrAlp++;                             \
        } while(0)
#elif (0u != CAPSENSE_PROX_RC_FILTER_EN)
    #define CAPSENSE_INC_PROX_FLTR_OBJ(fltrVariant)     \
        do {                                                    \
            (fltrVariant).ptrProx++;                            \
        } while(0)
#else
    #define CAPSENSE_INC_PROX_FLTR_OBJ(fltrVariant)     \
        do {                                                    \
        } while(0)
#endif

/*******************************************************************************
* Increments the pointer to the Filter History Object Variant
*******************************************************************************/
#define CAPSENSE_INC_FLTR_OBJ_VARIANT(isProxHistObj, fltrVariant)   \
    do {                                                                    \
        if (0u == (isProxHistObj))                                          \
        {                                                                   \
            CAPSENSE_INC_REG_FLTR_OBJ(fltrVariant);                 \
        }                                                                   \
        else                                                                \
        {                                                                   \
            CAPSENSE_INC_PROX_FLTR_OBJ(fltrVariant);                \
        }                                                                   \
    } while(0)

/*******************************************************************************
* Gets a widget status in the global enable register dsRam.wdgtEnable[]
*******************************************************************************/
#if (CAPSENSE_ENABLE == CAPSENSE_SELF_TEST_EN)
    #define CAPSENSE_GET_WIDGET_EN_STATUS(wdId)                                                     \
                 (CAPSENSE_GET_WDGT_STATUS_MASK(wdId) &                                             \
                  CAPSENSE_dsRam.wdgtEnable[CAPSENSE_GET_WDGT_STATUS_INDEX(wdId)] &         \
                  CAPSENSE_dsRam.wdgtWorking[CAPSENSE_GET_WDGT_STATUS_INDEX(wdId)])
#else
    #define CAPSENSE_GET_WIDGET_EN_STATUS(wdId)                    \
                (CAPSENSE_GET_WDGT_STATUS_MASK(wdId) &             \
                 CAPSENSE_dsRam.wdgtEnable[CAPSENSE_GET_WDGT_STATUS_INDEX(wdId)])
#endif /* (CAPSENSE_ENABLE == CAPSENSE_SELF_TEST_EN) */

/*******************************************************************************
* Gets a widget active status from the dsRam.wdgtStatus[] register
*******************************************************************************/
#define CAPSENSE_GET_WIDGET_ACTIVE_STATUS(wdId)                    \
            (CAPSENSE_GET_WDGT_STATUS_MASK(wdId) &                 \
             CAPSENSE_dsRam.wdgtStatus[CAPSENSE_GET_WDGT_STATUS_INDEX(wdId)])

/*******************************************************************************
* Declares Flash and RAM Data Structure variables
*******************************************************************************/
extern CAPSENSE_RAM_STRUCT            CAPSENSE_dsRam;
extern const CAPSENSE_FLASH_STRUCT    CAPSENSE_dsFlash;
extern const CAPSENSE_FLASH_IO_STRUCT CAPSENSE_ioList[CAPSENSE_TOTAL_ELECTRODES];
extern const CAPSENSE_RAM_WD_LIST_STRUCT CAPSENSE_ramWidgetInit;
extern const uint8 CAPSENSE_ramIdacInit[CAPSENSE_TOTAL_SENSORS];






/*******************************************************************************
* Function Prototypes
*******************************************************************************/

/*******************************************************************************
* High-level API
*******************************************************************************/

/**
* \cond SECTION_CYSENSE_HIGH_LEVEL
* \addtogroup group_cysense_high_level
* \{
*/

uint32 CAPSENSE_IsAnyWidgetActive(void);
uint32 CAPSENSE_IsWidgetActive(uint32 widgetId);
uint32 CAPSENSE_IsSensorActive(uint32 widgetId, uint32 sensorId);

#if (0u != CAPSENSE_MATRIX_WIDGET_EN)
    uint32 CAPSENSE_IsMatrixButtonsActive(uint32 widgetId);
#endif /* #if (0u != CAPSENSE_MATRIX_WIDGET_EN) */

#if (0u != CAPSENSE_PROXIMITY_WIDGET_EN)
    uint32 CAPSENSE_IsProximitySensorActive(uint32 widgetId, uint32 proxId);
#endif /* #if (0u != CAPSENSE_PROXIMITY_WIDGET_EN) */

#if (0u != CAPSENSE_SLIDER_WIDGET_EN)
    uint32 CAPSENSE_GetCentroidPos(uint32 widgetId);
#endif /* #if (0u != CAPSENSE_SLIDER_WIDGET_EN) */

#if (0u != CAPSENSE_TOUCHPAD_WIDGET_EN)
    uint32 CAPSENSE_GetXYCoordinates(uint32 widgetId);
#endif /* #if (0u != CAPSENSE_TOUCHPAD_WIDGET_EN) */

/** \}
* \endcond */

/*******************************************************************************
* Low level API
*******************************************************************************/

/**
* \cond SECTION_CYSENSE_LOW_LEVEL
* \addtogroup group_cysense_low_level
* \{
*/

cy_status CAPSENSE_GetParam(uint32 paramId, uint32 *value);
cy_status CAPSENSE_SetParam(uint32 paramId, uint32 value);

/** \}
* \endcond */

/*******************************************************************************
* Function Prototypes - internal functions
*******************************************************************************/
/**
* \cond SECTION_CYSENSE_INTERNAL
* \addtogroup group_cysense_internal
* \{
*/



void CAPSENSE_DsInitialize(void);
#if (0u != CAPSENSE_ADC_EN)
    void CAPSENSE_AdcDsInitialize(void);
#endif /* (0u != CAPSENSE_ADC_EN) */

/** \}
* \endcond */

#endif /* End CY_SENSE_CAPSENSE_STRUCTURE_H */

/* [] END OF FILE */
