/***************************************************************************//**
* \file CAPSENSE_INT.c
* \version 2.0
*
* \brief
*   This file contains the source code for implementation of the CAPSENSE
*   Component Interrupt Service Routine (ISR).
*
* \see CAPSENSE v2.0 Datasheet
*
*//*****************************************************************************
* Copyright (2016-2017), Cypress Semiconductor Corporation.
********************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress) and is
* protected by and subject to worldwide patent protection (United States and
* foreign), United States copyright laws and international treaty provisions.
* Cypress hereby grants to licensee a personal, non-exclusive, non-transferable
* license to copy, use, modify, create derivative works of, and compile the
* Cypress Source Code and derivative works for the sole purpose of creating
* custom software in support of licensee product to be used only in conjunction
* with a Cypress integrated circuit as specified in the applicable agreement.
* Any reproduction, modification, translation, compilation, or representation of
* this software except as specified above is prohibited without the express
* written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the
* materials described herein. Cypress does not assume any liability arising out
* of the application or use of any product or circuit described herein. Cypress
* does not authorize its products for use as critical components in life-support
* systems where a malfunction or failure may reasonably be expected to result in
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of
* such use and in doing so indemnifies Cypress against all charges. Use may be
* limited by and subject to the applicable Cypress software license agreement.
*******************************************************************************/
#include "syslib/cy_syslib.h"
#include "cyfitter.h"
#include "CAPSENSE_Configuration.h"
#include "CAPSENSE_Structure.h"
#include "CAPSENSE_Sensing.h"
#if (CAPSENSE_ENABLE == CAPSENSE_CSD_EN)
    #include "CAPSENSE_SensingCSD_LL.h"
#endif /* (CAPSENSE_ENABLE == CAPSENSE_CSD_EN) */
#include "cyapicallbacks.h"

/*******************************************************************************
* Static Function Prototypes
*******************************************************************************/
/**
* \cond SECTION_CYSENSE_INTERNAL
* \addtogroup group_cysense_internal
* \{
*/

#if (((CAPSENSE_ENABLE == CAPSENSE_CSD_EN) || (CAPSENSE_ENABLE == CAPSENSE_CSD_CSX_EN)) && \
     (CAPSENSE_ENABLE == CAPSENSE_MULTI_FREQ_SCAN_EN))
    static void CAPSENSE_SsNextFrequencyScan(void);
#endif /* (((CAPSENSE_ENABLE == CAPSENSE_CSD_EN) || (CAPSENSE_ENABLE == CAPSENSE_CSD_CSX_EN)) && \
            (CAPSENSE_ENABLE == CAPSENSE_MULTI_FREQ_SCAN_EN)) */

#if ((CAPSENSE_ENABLE == CAPSENSE_CSD_EN) || (CAPSENSE_ENABLE == CAPSENSE_CSD_CSX_EN))
    static void CAPSENSE_SsCSDPostScan(void);
    static void CAPSENSE_SsCSDInitNextScan(void);
#endif /* ((CAPSENSE_ENABLE == CAPSENSE_CSD_EN) || (CAPSENSE_ENABLE == CAPSENSE_CSD_CSX_EN)) */
/** \}
* \endcond */


/**
* \cond SECTION_CYSENSE_INTERRUPT
* \addtogroup group_cysense_interrupt
* \{
*/


#if ((CAPSENSE_ENABLE == CAPSENSE_CSD_EN) || (CAPSENSE_ENABLE == CAPSENSE_CSD_CSX_EN))


    /*******************************************************************************
    * Function Name: CAPSENSE_CSDPostSingleScan
    ****************************************************************************//**
    *
    * \brief
    *  This is an internal ISR function for the single-sensor scanning implementation.
    *
    * \details
    *  This ISR handler is triggered when the user calls the
    *  CAPSENSE_CSDScanExt() function.
    *
    *  The following tasks are performed:
    *    1. Check if the raw data is not noisy.
    *    2. Read the Counter register and update the data structure with raw data.
    *    3. Configure and start the scan for the next frequency if the
    *      multi-frequency is enabled.
    *    4. Update the Scan Counter.
    *    5. Reset the BUSY flag.
    *    6. Enable the CSD interrupt.
    *
    *  The ISR handler changes the scanning for the next frequency
    *  channels when multi-frequency scanning is enabled.
    *
    *  This function has two Macro Callbacks that allow calling the user code
    *  from macros specified in Component's generated code. Refer to the
    *  \ref group_cysense_macrocallbacks section of the PSoC Creator User Guide
    *  for details.
    *
    *******************************************************************************/
    void CAPSENSE_CSDPostSingleScan(void)
    {
        #ifdef CAPSENSE_ENTRY_CALLBACK
            CAPSENSE_EntryCallback();
        #endif /* CAPSENSE_ENTRY_CALLBACK */

        /* Clear pending interrupt */
        CY_SET_REG32(CAPSENSE_CSD_INTR_PTR, CAPSENSE_CSD_INTR_ALL_MSK);
        (void)CY_GET_REG32(CAPSENSE_CSD_INTR_PTR);

        #if (CAPSENSE_ENABLE == CAPSENSE_CSD_NOISE_METRIC_EN)
            if ((CAPSENSE_CSD_NOISE_METRIC_TH < ((CY_GET_REG32(CAPSENSE_CSD_RESULT_VAL1_PTR) &
                                                        CAPSENSE_CSD_RESULT_VAL1_BAD_CONVS_MSK) >>
                                                        CAPSENSE_CSD_RESULT_VAL1_BAD_CONVS_POS)) &&
                                                        (0u < CAPSENSE_badConversionsNum))
            {
                /* Decrement bad conversions number */
                CAPSENSE_badConversionsNum--;

                /* Start the re-scan */
                CY_SET_REG32(CAPSENSE_CSD_SEQ_START_PTR, CAPSENSE_CSD_SEQ_START_AZ0_SKIP_MSK |
                                                             CAPSENSE_CSD_SEQ_START_AZ1_SKIP_MSK |
                                                             CAPSENSE_CSD_SEQ_START_START_MSK);
            }
            else
            {
        #endif /* (CAPSENSE_ENABLE == CAPSENSE_CSD_NOISE_METRIC_EN) */

            CAPSENSE_SsCSDPostScan();

            #if (CAPSENSE_ENABLE == CAPSENSE_MULTI_FREQ_SCAN_EN)
                if (CAPSENSE_FREQ_CHANNEL_2 > CAPSENSE_scanFreqIndex)
                {
                    /* Scan the next channel */
                    CAPSENSE_SsNextFrequencyScan();
                }
                else
                {
                    /* All channels are scanned. Reset the frequency scan channel */
                    CAPSENSE_scanFreqIndex = CAPSENSE_FREQ_CHANNEL_0;
                    CAPSENSE_SsChangeClkFreq(CAPSENSE_FREQ_CHANNEL_0);

                    #if (CAPSENSE_ENABLE == CAPSENSE_BLOCK_OFF_AFTER_SCAN_EN)
                        /* Disable HW IP block */
                        CY_SET_REG32(CAPSENSE_CSD_CONFIG_PTR, CAPSENSE_configCsd);
                    #endif /* (CAPSENSE_ENABLE == CAPSENSE_BLOCK_OFF_AFTER_SCAN_EN) */

                    /* Update Scan Counter */
                    CAPSENSE_dsRam.scanCounter++;

                    /* Sensor is totally scanned. Reset BUSY flag */
                    CAPSENSE_dsRam.status &= ~(CAPSENSE_SW_STS_BUSY | CAPSENSE_WDGT_SW_STS_BUSY);
                }
            #else
                {
                    #if (CAPSENSE_ENABLE == CAPSENSE_BLOCK_OFF_AFTER_SCAN_EN)
                        /* Disable HW IP block */
                        CY_SET_REG32(CAPSENSE_CSD_CONFIG_PTR, CAPSENSE_configCsd);
                    #endif /* (CAPSENSE_ENABLE == CAPSENSE_BLOCK_OFF_AFTER_SCAN_EN) */

                    /* Update Scan Counter */
                    CAPSENSE_dsRam.scanCounter++;

                    /* Sensor is totally scanned. Reset BUSY flag */
                    CAPSENSE_dsRam.status &= ~(CAPSENSE_SW_STS_BUSY | CAPSENSE_WDGT_SW_STS_BUSY);
                }
            #endif /* (CAPSENSE_ENABLE == CAPSENSE_MULTI_FREQ_SCAN_EN) */
    #if (CAPSENSE_ENABLE == CAPSENSE_CSD_NOISE_METRIC_EN)
        }
    #endif /* (CAPSENSE_ENABLE == CAPSENSE_CSD_NOISE_METRIC_EN) */

        #ifdef CAPSENSE_EXIT_CALLBACK
            CAPSENSE_ExitCallback();
        #endif /* CAPSENSE_EXIT_CALLBACK */
    }


    /*******************************************************************************
    * Function Name: CAPSENSE_CSDPostMultiScan
    ****************************************************************************//**
    *
    * \brief
    *  This is an internal ISR function for the multiple-sensor scanning
    *  implementation.
    *
    * \details
    *  This ISR handler is triggered when the user calls the
    *  CAPSENSE_Scan() or CAPSENSE_ScanAllWidgets() APIs.
    *
    *  The following tasks are performed:
    *    1. Disable the CSD interrupt.
    *    2. Read the Counter register and update the data structure with raw data.
    *    3. Connect the Vref buffer to the AMUX bus.
    *    4. Disable the CSD block (after the widget has been scanned).
    *    5. Update the Scan Counter.
    *    6. Reset the BUSY flag.
    *    7. Enable the CSD interrupt.
    *
    *  The ISR handler initializes scanning for the previous sensor when the
    *  widget has more than one sensor.
    *  The ISR handler initializes scanning for the next widget when the
    *  CAPSENSE_ScanAllWidgets() APIs are called and the project has
    *  more than one widget.
    *  The ISR handler changes the IMO and initializes scanning for the next
    *  frequency channels when multi-frequency scanning is enabled.
    *
    *  This function has two Macro Callbacks that allow calling the user
    *  code from macros specified in Component's generated code. Refer to the
    *  \ref group_cysense_macrocallbacks section of the PSoC Creator User Guide
    *  for details.
    *
    *******************************************************************************/
    void CAPSENSE_CSDPostMultiScan(void)
    {
        /* Declare and initialize ptr to sensor IO structure to appropriate address */
        CAPSENSE_FLASH_IO_STRUCT const *curSnsIOPtr = (CAPSENSE_FLASH_IO_STRUCT const *)
                                                          CAPSENSE_dsFlash.wdgtArray[CAPSENSE_widgetIndex].ptr2SnsFlash
                                                          + CAPSENSE_sensorIndex;

        #ifdef CAPSENSE_ENTRY_CALLBACK
            CAPSENSE_EntryCallback();
        #endif /* CAPSENSE_ENTRY_CALLBACK */

        /* Clear pending interrupt */
        CY_SET_REG32(CAPSENSE_CSD_INTR_PTR, CAPSENSE_CSD_INTR_ALL_MSK);
        (void)CY_GET_REG32(CAPSENSE_CSD_INTR_PTR);

        #if (CAPSENSE_ENABLE == CAPSENSE_CSD_NOISE_METRIC_EN)
            if ((CAPSENSE_CSD_NOISE_METRIC_TH < ((CY_GET_REG32(CAPSENSE_CSD_RESULT_VAL1_PTR) &
                                                      CAPSENSE_CSD_RESULT_VAL1_BAD_CONVS_MSK) >>
                                                      CAPSENSE_CSD_RESULT_VAL1_BAD_CONVS_POS)) &&
                                                      (0u < CAPSENSE_badConversionsNum))
            {
                /* Decrement bad conversions number */
                CAPSENSE_badConversionsNum--;

                /* Start the re-scan */
                CY_SET_REG32(CAPSENSE_CSD_SEQ_START_PTR, CAPSENSE_CSD_SEQ_START_AZ0_SKIP_MSK |
                                                             CAPSENSE_CSD_SEQ_START_AZ1_SKIP_MSK |
                                                             CAPSENSE_CSD_SEQ_START_START_MSK);
            }
            else
            {
        #endif /* (CAPSENSE_ENABLE == CAPSENSE_CSD_NOISE_METRIC_EN) */

        CAPSENSE_SsCSDPostScan();

        /* Disable sensor when all frequency channels are scanned */
        #if (CAPSENSE_ENABLE == CAPSENSE_MULTI_FREQ_SCAN_EN)
            if (CAPSENSE_FREQ_CHANNEL_2 == CAPSENSE_scanFreqIndex)
        #endif /* (CAPSENSE_ENABLE == CAPSENSE_MULTI_FREQ_SCAN_EN) */
        {
            /* Disable sensor */
            CAPSENSE_CSDDisconnectSns(curSnsIOPtr);
        }

        #if (CAPSENSE_ENABLE == CAPSENSE_MULTI_FREQ_SCAN_EN)
            if (CAPSENSE_FREQ_CHANNEL_2 > CAPSENSE_scanFreqIndex)
            {
                 /* Scan the next channel */
                CAPSENSE_SsNextFrequencyScan();
            }
            else
            {
                /* All channels are scanned. Reset the frequency scan channel */
                CAPSENSE_scanFreqIndex = CAPSENSE_FREQ_CHANNEL_0;
                CAPSENSE_SsChangeClkFreq(CAPSENSE_FREQ_CHANNEL_0);

                 /* Scan the next sensor */
                CAPSENSE_SsCSDInitNextScan();
            }
        #else
            /* Scan the next sensor */
            CAPSENSE_SsCSDInitNextScan();
        #endif /* (CAPSENSE_ENABLE == CAPSENSE_MULTI_FREQ_SCAN_EN) */
    #if (CAPSENSE_ENABLE == CAPSENSE_CSD_NOISE_METRIC_EN)
        }
    #endif /* (CAPSENSE_ENABLE == CAPSENSE_CSD_NOISE_METRIC_EN) */

    #ifdef CAPSENSE_EXIT_CALLBACK
        CAPSENSE_ExitCallback();
    #endif /* CAPSENSE_EXIT_CALLBACK */
}


#if (CAPSENSE_ENABLE == CAPSENSE_CSD_GANGED_SNS_EN)
/*******************************************************************************
* Function Name: CAPSENSE_CSDPostMultiScanGanged
****************************************************************************//**
*
* \brief
*  This is an internal ISR function for the multiple-sensor scanning
*  implementation for ganged sensors.
*
* \details
*  This ISR handler is triggered when the user calls the
*  CAPSENSE_Scan() API for a ganged sensor or the
*  CAPSENSE_ScanAllWidgets() API in the project with ganged sensors.
*
*  The following tasks are performed:
*    1. Disable the CSD interrupt.
*    2. Read the Counter register and update the data structure with raw data.
*    3. Connect the Vref buffer to the AMUX bus.
*    4. Disable the CSD block (after the widget has been scanned).
*    5. Update the Scan Counter.
*    6. Reset the BUSY flag.
*    7. Enable the CSD interrupt.
*
*  The ISR handler initializes scanning for the previous sensor when the
*  widget has more than one sensor.
*  The ISR handler initializes scanning for the next widget when the
*  CAPSENSE_ScanAllWidgets() APIs are called and the project has
*  more than one widget.
*  The ISR handler changes the IMO and initializes scanning for the next
*  frequency channels when multi-frequency scanning is enabled.
*
*  This function has two Macro Callbacks that allow calling the user
*  code from macros specified in Component's generated code. Refer to the
*  \ref group_cysense_macrocallbacks section of the PSoC Creator User Guide
*  for details.
*
*******************************************************************************/
void CAPSENSE_CSDPostMultiScanGanged(void)
{
    #ifdef CAPSENSE_ENTRY_CALLBACK
        CAPSENSE_EntryCallback();
    #endif /* CAPSENSE_ENTRY_CALLBACK */

        /* Clear pending interrupt */
    CY_SET_REG32(CAPSENSE_CSD_INTR_PTR, CAPSENSE_CSD_INTR_ALL_MSK);
    (void)CY_GET_REG32(CAPSENSE_CSD_INTR_PTR);

    #if (CAPSENSE_ENABLE == CAPSENSE_CSD_NOISE_METRIC_EN)
        if ((CAPSENSE_CSD_NOISE_METRIC_TH < ((CY_GET_REG32(CAPSENSE_CSD_RESULT_VAL1_PTR) &
                                                  CAPSENSE_CSD_RESULT_VAL1_BAD_CONVS_MSK) >>
                                                  CAPSENSE_CSD_RESULT_VAL1_BAD_CONVS_POS)) &&
                                                  (0u < CAPSENSE_badConversionsNum))
        {
            /* Decrement bad conversions number */
            CAPSENSE_badConversionsNum--;

            /* Start the re-scan */
            CY_SET_REG32(CAPSENSE_CSD_SEQ_START_PTR, CAPSENSE_CSD_SEQ_START_AZ0_SKIP_MSK |
                                                         CAPSENSE_CSD_SEQ_START_AZ1_SKIP_MSK |
                                                         CAPSENSE_CSD_SEQ_START_START_MSK);
        }
        else
        {
    #endif /* (CAPSENSE_ENABLE == CAPSENSE_CSD_NOISE_METRIC_EN) */

        CAPSENSE_SsCSDPostScan();

        #if (CAPSENSE_ENABLE == CAPSENSE_MULTI_FREQ_SCAN_EN)
            if (CAPSENSE_FREQ_CHANNEL_2 == CAPSENSE_scanFreqIndex)
        #endif /* (CAPSENSE_ENABLE == CAPSENSE_MULTI_FREQ_SCAN_EN) */
        {
            CAPSENSE_SsCSDDisconnectSnsExt((uint32)CAPSENSE_widgetIndex, (uint32)CAPSENSE_sensorIndex);
        }

        #if (CAPSENSE_ENABLE == CAPSENSE_MULTI_FREQ_SCAN_EN)
            if (CAPSENSE_FREQ_CHANNEL_2 > CAPSENSE_scanFreqIndex)
            {
                 /* Scan the next channel */
                CAPSENSE_SsNextFrequencyScan();
            }
            else
            {
                /* All channels are scanned. Reset the frequency scan channel */
                CAPSENSE_scanFreqIndex = CAPSENSE_FREQ_CHANNEL_0;
                CAPSENSE_SsChangeClkFreq(CAPSENSE_FREQ_CHANNEL_0);

                 /* Scan the next sensor */
                CAPSENSE_SsCSDInitNextScan();
            }
        #else
             /* Scan the next sensor */
            CAPSENSE_SsCSDInitNextScan();
        #endif /* (CAPSENSE_ENABLE == CAPSENSE_MULTI_FREQ_SCAN_EN) */
    #if (CAPSENSE_ENABLE == CAPSENSE_CSD_NOISE_METRIC_EN)
        }
    #endif /* (CAPSENSE_ENABLE == CAPSENSE_CSD_NOISE_METRIC_EN) */

    #ifdef CAPSENSE_EXIT_CALLBACK
        CAPSENSE_ExitCallback();
    #endif /* CAPSENSE_EXIT_CALLBACK */

    (void)CY_GET_REG32(CAPSENSE_CSD_INTR_PTR);
}
#endif /* (CAPSENSE_ENABLE == CAPSENSE_CSD_GANGED_SNS_EN) */


#endif /* ((CAPSENSE_ENABLE == CAPSENSE_CSD_EN) || (CAPSENSE_ENABLE == CAPSENSE_CSD_CSX_EN)) */

/** \}
 * \endcond */


#if ((CAPSENSE_ENABLE == CAPSENSE_CSD_EN) || (CAPSENSE_ENABLE == CAPSENSE_CSD_CSX_EN))

/*******************************************************************************
* Function Name: CAPSENSE_SsCSDPostScan
****************************************************************************//**
*
* \brief
*   This function reads rawdata and releases required HW resources after scan.
*
* \details
*   This function performs following tasks after scan:
*   - Reads SlotResult from Raw Counter;
*   - Inits bad Conversions number;
*   - Disconnects Vrefhi from AMUBUF positive input;
*   - Disconnects AMUBUF output from CSDBUSB with sych PHI2+HSCMP;
*   - Opens HCBV and HCBG switches.
*
*******************************************************************************/
static void CAPSENSE_SsCSDPostScan(void)
{
    uint32 tmpRawData;
    uint32 tmpMaxCount;
    CAPSENSE_RAM_WD_BASE_STRUCT const *ptrWdgt = (CAPSENSE_RAM_WD_BASE_STRUCT *)
                                        CAPSENSE_dsFlash.wdgtArray[CAPSENSE_widgetIndex].ptr2WdgtRam;

    /* Read SlotResult from Raw Counter */
    tmpRawData = CAPSENSE_CSD_RESULT_VAL1_REG & CAPSENSE_CSD_RESULT_VAL1_VALUE_MSK;

    tmpMaxCount = ((1uL << ptrWdgt->resolution) - 1uL);
    if(tmpRawData < tmpMaxCount)
    {
        CAPSENSE_curRamSnsPtr->raw[CAPSENSE_scanFreqIndex] = CY_LO16(tmpRawData);
    }
    else
    {
        CAPSENSE_curRamSnsPtr->raw[CAPSENSE_scanFreqIndex] = CY_LO16(tmpMaxCount);
    }

    #if (CAPSENSE_ENABLE == CAPSENSE_CSD_NOISE_METRIC_EN)
        /* Init bad Conversions number */
        CAPSENSE_badConversionsNum = CAPSENSE_BAD_CONVERSIONS_NUM;
    #endif /* (CAPSENSE_ENABLE == CAPSENSE_CSD_NOISE_METRIC_EN) */

    #if (CAPSENSE_ENABLE == CAPSENSE_CSD_SHIELD_EN)
        /* Open HCBV and HCBG switches */
        CY_SET_REG32(CAPSENSE_CSD_SW_SHIELD_SEL_PTR, CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCBV_STATIC_OPEN |
                                                         CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCBG_STATIC_OPEN);
    #endif /* (CAPSENSE_ENABLE == CAPSENSE_CSD_SHIELD_EN) */
}


/*******************************************************************************
* Function Name: CAPSENSE_SsCSDInitNextScan
****************************************************************************//**
*
* \brief
*   This function initializes the next sensor scan.
*
* \details
*   The function increments the sensor index, updates sense clock for matrix
*   or touchpad widgets only, sets up Compensation IDAC, enables the sensor and
*   scans it. When all the sensors are scanned it continues to set up the next widget
*   until all the widgets are scanned. The CSD block is disabled when all the widgets are
*   scanned.
*
*******************************************************************************/
static void CAPSENSE_SsCSDInitNextScan(void)
{
    /* Declare and initialize ptr to widget and sensor structures to appropriate address */
    #if (((CAPSENSE_ENABLE == CAPSENSE_CSD_IDAC_COMP_EN) || \
            (CAPSENSE_CSD_MATRIX_WIDGET_EN || CAPSENSE_CSD_TOUCHPAD_WIDGET_EN)) || \
            (((CAPSENSE_DISABLE == CAPSENSE_CSD_COMMON_SNS_CLK_EN) && \
            (CAPSENSE_CSD_MATRIX_WIDGET_EN || CAPSENSE_CSD_TOUCHPAD_WIDGET_EN))))
        CAPSENSE_RAM_WD_BASE_STRUCT *ptrWdgt = (CAPSENSE_RAM_WD_BASE_STRUCT *)
            CAPSENSE_dsFlash.wdgtArray[CAPSENSE_widgetIndex].ptr2WdgtRam;
    #endif

    /* Check if all the sensors are scanned in widget */
    if (((uint8)CAPSENSE_dsFlash.wdgtArray[(CAPSENSE_widgetIndex)].totalNumSns - 1u) > CAPSENSE_sensorIndex)
    {
        /* Increment sensor index to configure next sensor in widget */
        CAPSENSE_sensorIndex++;

        /* Update global pointer to CAPSENSE_RAM_SNS_STRUCT to current sensor */
        CAPSENSE_curRamSnsPtr = (CAPSENSE_RAM_SNS_STRUCT *)
                                                  CAPSENSE_dsFlash.wdgtArray[CAPSENSE_widgetIndex].ptr2SnsRam
                                                  + CAPSENSE_sensorIndex;

        /* Configure clock divider to row or column */
        #if ((CAPSENSE_DISABLE == CAPSENSE_CSD_COMMON_SNS_CLK_EN) && \
             (CAPSENSE_CSD_MATRIX_WIDGET_EN || CAPSENSE_CSD_TOUCHPAD_WIDGET_EN))
            if ((CAPSENSE_WD_TOUCHPAD_E == (CAPSENSE_WD_TYPE_ENUM)CAPSENSE_dsFlash.wdgtArray[(CAPSENSE_widgetIndex)].wdgtType) ||
                (CAPSENSE_WD_MATRIX_BUTTON_E == (CAPSENSE_WD_TYPE_ENUM)CAPSENSE_dsFlash.wdgtArray[(CAPSENSE_widgetIndex)].wdgtType))
            {
                CAPSENSE_SsCSDConfigClock();

                /* Set up scanning resolution */
                CAPSENSE_SsCSDCalculateScanDuration(ptrWdgt);
            }
        #endif /* ((CAPSENSE_DISABLE == CAPSENSE_CSD_COMMON_SNS_CLK_EN) && \
                   (CAPSENSE_CSD_MATRIX_WIDGET_EN || CAPSENSE_CSD_TOUCHPAD_WIDGET_EN))) */

        /* Setup Compensation IDAC for next sensor in widget */
        #if ((CAPSENSE_ENABLE == CAPSENSE_CSD_IDAC_COMP_EN) || \
             (CAPSENSE_CSD_MATRIX_WIDGET_EN || CAPSENSE_CSD_TOUCHPAD_WIDGET_EN))
            CAPSENSE_SsCSDSetUpIdacs(ptrWdgt);
        #endif /* ((CAPSENSE_ENABLE == CAPSENSE_CSD_IDAC_COMP_EN) || \
             (CAPSENSE_CSD_MATRIX_WIDGET_EN || CAPSENSE_CSD_TOUCHPAD_WIDGET_EN)) */

        /* Enable sensor */
        CAPSENSE_SsCSDConnectSensorExt((uint32)CAPSENSE_widgetIndex, (uint32)CAPSENSE_sensorIndex);

        /* Proceed scanning */
        CAPSENSE_SsCSDStartSample();
    }
    /* Call scan next widget API if requested, if not, complete the scan */
    else
    {
        CAPSENSE_sensorIndex = 0u;

        /* Current widget is totally scanned. Reset WIDGET BUSY flag */
        CAPSENSE_dsRam.status &= ~CAPSENSE_WDGT_SW_STS_BUSY;

        /* Check if all the widgets have been scanned */
        if (CAPSENSE_ENABLE == CAPSENSE_requestScanAllWidget)
        {
            /* Configure and begin scanning next widget */
            CAPSENSE_SsPostAllWidgetsScan();
        }
        else
        {
            #if (CAPSENSE_ENABLE == CAPSENSE_BLOCK_OFF_AFTER_SCAN_EN)
                /* Disable the CSD block */
                CY_SET_REG32(CAPSENSE_CSD_CONFIG_PTR, CAPSENSE_configCsd);
            #endif /* (CAPSENSE_ENABLE == CAPSENSE_BLOCK_OFF_AFTER_SCAN_EN) */

            /* all the widgets are totally scanned. Reset BUSY flag */
            CAPSENSE_dsRam.status &= ~CAPSENSE_SW_STS_BUSY;

            /* Update scan Counter */
            CAPSENSE_dsRam.scanCounter++;
        }
    }
}

#if (CAPSENSE_ENABLE == CAPSENSE_MULTI_FREQ_SCAN_EN)
    /*******************************************************************************
    * Function Name: CAPSENSE_SsNextFrequencyScan
    ****************************************************************************//**
    *
    * \brief
    *   This function scans the sensor on the next frequency channel.
    *
    * \details
    *   The function increments the frequency channel, changes scan frequency and 
    *   initializes the scanning process of the same sensor.
    *
    *******************************************************************************/
    static void CAPSENSE_SsNextFrequencyScan(void)
    {
        CAPSENSE_RAM_WD_BASE_STRUCT const *ptrWdgt = (CAPSENSE_RAM_WD_BASE_STRUCT *)
                                                        CAPSENSE_dsFlash.wdgtArray[CAPSENSE_widgetIndex].ptr2WdgtRam;

        CAPSENSE_scanFreqIndex++;

        /* Set Immunity */
        CAPSENSE_SsChangeClkFreq((uint32)CAPSENSE_scanFreqIndex);

        /* Update IDAC registers */
        CAPSENSE_SsCSDSetUpIdacs(ptrWdgt);

        /* Proceed scanning */
        CAPSENSE_SsCSDStartSample();
    }
#endif /* (CAPSENSE_ENABLE == CAPSENSE_MULTI_FREQ_SCAN_EN) */

#endif /* ((CAPSENSE_ENABLE == CAPSENSE_CSD_EN) || (CAPSENSE_ENABLE == CAPSENSE_CSD_CSX_EN)) */


/* [] END OF FILE */
