/***************************************************************************//**
* \file CAPSENSE_Sensing.h
* \version 2.0
*
* \brief
*   This file provides the headers of APIs specific to implementation of the
*   sensing FW block.
*
* \see CAPSENSE v2.0 Datasheet
*
*//*****************************************************************************
* Copyright (2016-2017), Cypress Semiconductor Corporation.
********************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress) and is
* protected by and subject to worldwide patent protection (United States and
* foreign), United States copyright laws and international treaty provisions.
* Cypress hereby grants to licensee a personal, non-exclusive, non-transferable
* license to copy, use, modify, create derivative works of, and compile the
* Cypress Source Code and derivative works for the sole purpose of creating
* custom software in support of licensee product to be used only in conjunction
* with a Cypress integrated circuit as specified in the applicable agreement.
* Any reproduction, modification, translation, compilation, or representation of
* this software except as specified above is prohibited without the express
* written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the
* materials described herein. Cypress does not assume any liability arising out
* of the application or use of any product or circuit described herein. Cypress
* does not authorize its products for use as critical components in life-support
* systems where a malfunction or failure may reasonably be expected to result in
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of
* such use and in doing so indemnifies Cypress against all charges. Use may be
* limited by and subject to the applicable Cypress software license agreement.
*******************************************************************************/

#if !defined(CY_SENSE_CAPSENSE_SENSING_H)
#define CY_SENSE_CAPSENSE_SENSING_H

#include "gpio/cy_gpio.h"
#include "cyfitter.h"
#include "syslib/cy_syslib.h"
#include "sysclk/cy_sysclk.h"
#include "CAPSENSE_Configuration.h"
#include "CAPSENSE_Structure.h"

#if (CY_CPU_CORTEX_M0P)
    #define CAPSENSE_CPU_CLOCK_FREQ_HZ  (CYDEV_CLK_SLOWCLK__HZ)
    #define CAPSENSE_CPU_CLOCK_FREQ_KHZ (CYDEV_CLK_SLOWCLK__KHZ)
    #define CAPSENSE_CPU_CLOCK_FREQ_MHZ (CYDEV_CLK_SLOWCLK__MHZ)
#else
    #define CAPSENSE_CPU_CLOCK_FREQ_HZ  (CYDEV_CLK_FASTCLK__HZ)
    #define CAPSENSE_CPU_CLOCK_FREQ_KHZ (CYDEV_CLK_FASTCLK__KHZ)
    #define CAPSENSE_CPU_CLOCK_FREQ_MHZ (CYDEV_CLK_FASTCLK__MHZ)
#endif

/*******************************************************************************
* Enumeration types definition
*******************************************************************************/

/* Pin states */
#define CAPSENSE_GROUND                                     (0u)
#define CAPSENSE_HIGHZ                                      (1u)
#define CAPSENSE_SHIELD                                     (2u)
#define CAPSENSE_SENSOR                                     (3u)
#define CAPSENSE_TX_PIN                                     (4u)
#define CAPSENSE_RX_PIN                                     (5u)

#if (CAPSENSE_ENABLE != CAPSENSE_ADC_STANDALONE_EN)
/*******************************************************************************
* Function Prototypes
*******************************************************************************/

/**
* \cond SECTION_CYSENSE_HIGH_LEVEL
* \addtogroup group_cysense_high_level
* \{
*/

cy_status CAPSENSE_SetupWidget(uint32 widgetId);
cy_status CAPSENSE_Scan(void);
cy_status CAPSENSE_ScanAllWidgets(void);
uint32 CAPSENSE_IsBusy(void);

/** \}
* \endcond */

/**
* \cond SECTION_CYSENSE_LOW_LEVEL
* \addtogroup group_cysense_low_level
* \{
*/

void CAPSENSE_SetPinState(uint32 widgetId, uint32 sensorElement, uint32 state);

#if ((CAPSENSE_ENABLE == CAPSENSE_CSD_IDAC_AUTOCAL_EN) || \
     (CAPSENSE_ENABLE == CAPSENSE_CSX_IDAC_AUTOCAL_EN))
cy_status CAPSENSE_CalibrateWidget(uint32 widgetId);
cy_status CAPSENSE_CalibrateAllWidgets(void);
#endif /* ((CAPSENSE_ENABLE == CAPSENSE_CSD_IDAC_AUTOCAL_EN) || \
           (CAPSENSE_ENABLE == CAPSENSE_CSX_IDAC_AUTOCAL_EN)) */

/** \}
* \endcond */

/**
* \cond SECTION_CYSENSE_INTERNAL
* \addtogroup group_cysense_internal
* \{
*/

cy_status CAPSENSE_SsInitialize(void);
void CAPSENSE_SsSetSnsClockDivider(uint32 snsClk);
void CAPSENSE_SsSetSnsFirstPhaseWidth(uint32 phaseWidth);
void CAPSENSE_SsSetClockDividers(uint32 snsClk, uint32 modClk);
void CAPSENSE_SsIsrInitialize(cy_israddress address);
void CAPSENSE_SsPostAllWidgetsScan(void);
void CAPSENSE_SsSetIOsInDefaultState(void);

#if ((CAPSENSE_ENABLE == CAPSENSE_CSD_CSX_EN) || \
     (CAPSENSE_ENABLE == CAPSENSE_SELF_TEST_EN) || \
     (CAPSENSE_ENABLE == CAPSENSE_ADC_EN))
    void CAPSENSE_SsSwitchSensingMode(CAPSENSE_SENSE_METHOD_ENUM mode);
#endif

#if (CAPSENSE_CSD_SS_DIS != CAPSENSE_CSD_AUTOTUNE)
    cy_status CAPSENSE_SsAutoTune(void);
#endif

#if (CAPSENSE_ENABLE == CAPSENSE_MULTI_FREQ_SCAN_EN)
    void CAPSENSE_SsChangeClkFreq(uint32 chId);
#endif

#if (CAPSENSE_ENABLE == CAPSENSE_ADC_EN)
    cy_status CAPSENSE_SsReleaseResources(void);
#endif

#if((CAPSENSE_ENABLE == CAPSENSE_CSD_EN) || \
    (CAPSENSE_ENABLE == CAPSENSE_CSX_EN))
    void CAPSENSE_SsInitializeSourceSenseClk(void);
#endif

#if (CAPSENSE_ENABLE == CAPSENSE_CSD_EN)
    void CAPSENSE_SsClearCSDSensors(void);
    uint32 CAPSENSE_SsCSDGetColSnsClkDivider(uint32 widgetId);
    void CAPSENSE_SsCSDSetColSnsClkDivider(uint32 widgetId, uint32 dividerVal);

    #if ((CAPSENSE_ENABLE == CAPSENSE_CSD_MATRIX_WIDGET_EN) || \
         (CAPSENSE_ENABLE == CAPSENSE_CSD_TOUCHPAD_WIDGET_EN))
        uint32 CAPSENSE_SsCSDGetRowSnsClkDivider(uint32 widgetId);
        void CAPSENSE_SsCSDSetRowSnsClkDivider(uint32 widgetId, uint32 dividerVal);
    #endif

    #if (CAPSENSE_CLK_SOURCE_PRSAUTO == CAPSENSE_CSD_SNS_CLK_SOURCE)
        uint8 CAPSENSE_SsCSDCalcPrsSize(uint32 snsClkDivider, uint32 resolution);
    #endif
#endif

void CAPSENSE_BistDischargeExtCapacitors(void);

/** \}
* \endcond */

/***************************************
* Global software/external variables
***************************************/
extern volatile uint8 CAPSENSE_widgetIndex;
extern volatile uint8 CAPSENSE_sensorIndex;
extern uint8 CAPSENSE_requestScanAllWidget;

extern CAPSENSE_RAM_SNS_STRUCT *CAPSENSE_curRamSnsPtr;

#if ((CAPSENSE_ENABLE == CAPSENSE_CSD_CSX_EN) || \
     (CAPSENSE_ENABLE == CAPSENSE_ADC_EN))
    extern CAPSENSE_SENSE_METHOD_ENUM CAPSENSE_currentSenseMethod;
#endif

#if(CAPSENSE_ENABLE == CAPSENSE_MULTI_FREQ_SCAN_EN)
    extern uint8 CAPSENSE_scanFreqIndex;
#else
    extern const uint8 CAPSENSE_scanFreqIndex;
#endif

#if ((CAPSENSE_ENABLE == CAPSENSE_CSD_GANGED_SNS_EN) || \
     (CAPSENSE_ENABLE == CAPSENSE_CSX_EN))
    extern CAPSENSE_FLASH_WD_STRUCT const *CAPSENSE_curFlashWdgtPtr;
#endif

#if (CAPSENSE_ENABLE == CAPSENSE_CSD_GANGED_SNS_EN)
    /* Pointer to Flash structure holding info of sensor to be scanned */
    extern CAPSENSE_FLASH_SNS_STRUCT const *CAPSENSE_curFlashSnsPtr;
#endif

extern CAPSENSE_FLASH_IO_STRUCT const *CAPSENSE_curSnsIOPtr;

#endif /* (CAPSENSE_ENABLE != CAPSENSE_ADC_STANDALONE_EN) */

/****************************************************************************
* HW CSD Block Definition
****************************************************************************/
#define CAPSENSE_CSD_HW                             ((CSD_Type*)CAPSENSE_CSD__HW)

#define CAPSENSE_CSD_CONFIG_REG                     ( CAPSENSE_CSD_HW->CONFIG)
#define CAPSENSE_CSD_CONFIG_PTR                     (&CAPSENSE_CSD_HW->CONFIG)
#define CAPSENSE_CSD_SPARE_REG                      ( CAPSENSE_CSD_HW->SPARE)
#define CAPSENSE_CSD_SPARE_PTR                      (&CAPSENSE_CSD_HW->SPARE)
#define CAPSENSE_CSD_STATUS_REG                     ( CAPSENSE_CSD_HW->STATUS)
#define CAPSENSE_CSD_STATUS_PTR                     (&CAPSENSE_CSD_HW->STATUS)
#define CAPSENSE_CSD_STAT_SEQ_REG                   ( CAPSENSE_CSD_HW->STAT_SEQ)
#define CAPSENSE_CSD_STAT_SEQ_PTR                   (&CAPSENSE_CSD_HW->STAT_SEQ)
#define CAPSENSE_CSD_STAT_CNTS_REG                  ( CAPSENSE_CSD_HW->STAT_CNTS)
#define CAPSENSE_CSD_STAT_CNTS_PTR                  (&CAPSENSE_CSD_HW->STAT_CNTS)
#define CAPSENSE_CSD_RESULT_VAL1_REG                ( CAPSENSE_CSD_HW->RESULT_VAL1)
#define CAPSENSE_CSD_RESULT_VAL1_PTR                (&CAPSENSE_CSD_HW->RESULT_VAL1)
#define CAPSENSE_CSD_RESULT_VAL2_REG                ( CAPSENSE_CSD_HW->RESULT_VAL2)
#define CAPSENSE_CSD_RESULT_VAL2_PTR                (&CAPSENSE_CSD_HW->RESULT_VAL2)
#define CAPSENSE_CSD_ADC_RES_REG                    ( CAPSENSE_CSD_HW->ADC_RES)
#define CAPSENSE_CSD_ADC_RES_PTR                    (&CAPSENSE_CSD_HW->ADC_RES)
#define CAPSENSE_CSD_INTR_REG                       ( CAPSENSE_CSD_HW->INTR)
#define CAPSENSE_CSD_INTR_PTR                       (&CAPSENSE_CSD_HW->INTR)
#define CAPSENSE_CSD_INTR_SET_REG                   ( CAPSENSE_CSD_HW->INTR_SET)
#define CAPSENSE_CSD_INTR_SET_PTR                   (&CAPSENSE_CSD_HW->INTR_SET)
#define CAPSENSE_CSD_INTR_MASK_REG                  ( CAPSENSE_CSD_HW->INTR_MASK)
#define CAPSENSE_CSD_INTR_MASK_PTR                  (&CAPSENSE_CSD_HW->INTR_MASK)
#define CAPSENSE_CSD_INTR_MASKED_REG                ( CAPSENSE_CSD_HW->INTR_MASKED)
#define CAPSENSE_CSD_INTR_MASKED_PTR                (&CAPSENSE_CSD_HW->INTR_MASKED)
#define CAPSENSE_CSD_HSCMP_REG                      ( CAPSENSE_CSD_HW->HSCMP)
#define CAPSENSE_CSD_HSCMP_PTR                      (&CAPSENSE_CSD_HW->HSCMP)
#define CAPSENSE_CSD_AMBUF_REG                      ( CAPSENSE_CSD_HW->AMBUF)
#define CAPSENSE_CSD_AMBUF_PTR                      (&CAPSENSE_CSD_HW->AMBUF)
#define CAPSENSE_CSD_REFGEN_REG                     ( CAPSENSE_CSD_HW->REFGEN)
#define CAPSENSE_CSD_REFGEN_PTR                     (&CAPSENSE_CSD_HW->REFGEN)
#define CAPSENSE_CSD_CSDCMP_REG                     ( CAPSENSE_CSD_HW->CSDCMP)
#define CAPSENSE_CSD_CSDCMP_PTR                     (&CAPSENSE_CSD_HW->CSDCMP)
#define CAPSENSE_CSD_SW_RES_REG                     ( CAPSENSE_CSD_HW->SW_RES)
#define CAPSENSE_CSD_SW_RES_PTR                     (&CAPSENSE_CSD_HW->SW_RES)
#define CAPSENSE_CSD_SENSE_PERIOD_REG               ( CAPSENSE_CSD_HW->SENSE_PERIOD)
#define CAPSENSE_CSD_SENSE_PERIOD_PTR               (&CAPSENSE_CSD_HW->SENSE_PERIOD)
#define CAPSENSE_CSD_SENSE_DUTY_REG                 ( CAPSENSE_CSD_HW->SENSE_DUTY)
#define CAPSENSE_CSD_SENSE_DUTY_PTR                 (&CAPSENSE_CSD_HW->SENSE_DUTY)
#define CAPSENSE_CSD_SW_HS_P_SEL_REG                ( CAPSENSE_CSD_HW->SW_HS_P_SEL)
#define CAPSENSE_CSD_SW_HS_P_SEL_PTR                (&CAPSENSE_CSD_HW->SW_HS_P_SEL)
#define CAPSENSE_CSD_SW_HS_N_SEL_REG                ( CAPSENSE_CSD_HW->SW_HS_N_SEL)
#define CAPSENSE_CSD_SW_HS_N_SEL_PTR                (&CAPSENSE_CSD_HW->SW_HS_N_SEL)
#define CAPSENSE_CSD_SW_SHIELD_SEL_REG              ( CAPSENSE_CSD_HW->SW_SHIELD_SEL)
#define CAPSENSE_CSD_SW_SHIELD_SEL_PTR              (&CAPSENSE_CSD_HW->SW_SHIELD_SEL)
#define CAPSENSE_CSD_SW_AMUXBUF_SEL_REG             ( CAPSENSE_CSD_HW->SW_AMUXBUF_SEL)
#define CAPSENSE_CSD_SW_AMUXBUF_SEL_PTR             (&CAPSENSE_CSD_HW->SW_AMUXBUF_SEL)
#define CAPSENSE_CSD_SW_BYP_SEL_REG                 ( CAPSENSE_CSD_HW->SW_BYP_SEL)
#define CAPSENSE_CSD_SW_BYP_SEL_PTR                 (&CAPSENSE_CSD_HW->SW_BYP_SEL)
#define CAPSENSE_CSD_SW_CMP_P_SEL_REG               ( CAPSENSE_CSD_HW->SW_CMP_P_SEL)
#define CAPSENSE_CSD_SW_CMP_P_SEL_PTR               (&CAPSENSE_CSD_HW->SW_CMP_P_SEL)
#define CAPSENSE_CSD_SW_CMP_N_SEL_REG               ( CAPSENSE_CSD_HW->SW_CMP_N_SEL)
#define CAPSENSE_CSD_SW_CMP_N_SEL_PTR               (&CAPSENSE_CSD_HW->SW_CMP_N_SEL)
#define CAPSENSE_CSD_SW_REFGEN_SEL_REG              ( CAPSENSE_CSD_HW->SW_REFGEN_SEL)
#define CAPSENSE_CSD_SW_REFGEN_SEL_PTR              (&CAPSENSE_CSD_HW->SW_REFGEN_SEL)
#define CAPSENSE_CSD_SW_FW_MOD_SEL_REG              ( CAPSENSE_CSD_HW->SW_FW_MOD_SEL)
#define CAPSENSE_CSD_SW_FW_MOD_SEL_PTR              (&CAPSENSE_CSD_HW->SW_FW_MOD_SEL)
#define CAPSENSE_CSD_SW_FW_TANK_SEL_REG             ( CAPSENSE_CSD_HW->SW_FW_TANK_SEL)
#define CAPSENSE_CSD_SW_FW_TANK_SEL_PTR             (&CAPSENSE_CSD_HW->SW_FW_TANK_SEL)
#define CAPSENSE_CSD_SW_DSI_SEL_REG                 ( CAPSENSE_CSD_HW->SW_DSI_SEL)
#define CAPSENSE_CSD_SW_DSI_SEL_PTR                 (&CAPSENSE_CSD_HW->SW_DSI_SEL)
#define CAPSENSE_CSD_IO_SEL_REG                     ( CAPSENSE_CSD_HW->IO_SEL)
#define CAPSENSE_CSD_IO_SEL_PTR                     (&CAPSENSE_CSD_HW->IO_SEL)
#define CAPSENSE_CSD_SEQ_TIME_REG                   ( CAPSENSE_CSD_HW->SEQ_TIME)
#define CAPSENSE_CSD_SEQ_TIME_PTR                   (&CAPSENSE_CSD_HW->SEQ_TIME)
#define CAPSENSE_CSD_SEQ_INIT_CNT_REG               ( CAPSENSE_CSD_HW->SEQ_INIT_CNT)
#define CAPSENSE_CSD_SEQ_INIT_CNT_PTR               (&CAPSENSE_CSD_HW->SEQ_INIT_CNT)
#define CAPSENSE_CSD_SEQ_NORM_CNT_REG               ( CAPSENSE_CSD_HW->SEQ_NORM_CNT)
#define CAPSENSE_CSD_SEQ_NORM_CNT_PTR               (&CAPSENSE_CSD_HW->SEQ_NORM_CNT)
#define CAPSENSE_CSD_ADC_CTL_REG                    ( CAPSENSE_CSD_HW->ADC_CTL)
#define CAPSENSE_CSD_ADC_CTL_PTR                    (&CAPSENSE_CSD_HW->ADC_CTL)
#define CAPSENSE_CSD_SEQ_START_REG                  ( CAPSENSE_CSD_HW->SEQ_START)
#define CAPSENSE_CSD_SEQ_START_PTR                  (&CAPSENSE_CSD_HW->SEQ_START)
#define CAPSENSE_CSD_IDACA_REG                      ( CAPSENSE_CSD_HW->IDACA)
#define CAPSENSE_CSD_IDACA_PTR                      (&CAPSENSE_CSD_HW->IDACA)
#define CAPSENSE_CSD_IDACB_REG                      ( CAPSENSE_CSD_HW->IDACB)
#define CAPSENSE_CSD_IDACB_PTR                      (&CAPSENSE_CSD_HW->IDACB)

/****************************************************************************
* HW CSD Waveform Selection values
****************************************************************************/

#define CAPSENSE_CSD_WAVEFORM_STATIC_OPEN           (0x00000000uL)
#define CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED         (0x00000001uL)
#define CAPSENSE_CSD_WAVEFORM_PHI1                  (0x00000002uL)
#define CAPSENSE_CSD_WAVEFORM_PHI2                  (0x00000003uL)
#define CAPSENSE_CSD_WAVEFORM_PHI1_HSCMP            (0x00000004uL)
#define CAPSENSE_CSD_WAVEFORM_PHI2_HSCMP            (0x00000005uL)
#define CAPSENSE_CSD_WAVEFORM_HSCMP                 (0x00000006uL)
#define CAPSENSE_CSD_WAVEFORM_SENSE_INV             (0x00000007uL)
#define CAPSENSE_CSD_WAVEFORM_PHI1_DELAY            (0x00000008uL)
#define CAPSENSE_CSD_WAVEFORM_PHI2_DELAY            (0x00000009uL)
#define CAPSENSE_CSD_WAVEFORM_PHI1_INV              (0x0000000AuL)
#define CAPSENSE_CSD_WAVEFORM_PHI2_INV              (0x0000000BuL)
#define CAPSENSE_CSD_WAVEFORM_PHI1_HSCMP_INV        (0x0000000CuL)
#define CAPSENSE_CSD_WAVEFORM_PHI2_HSCMP_INV        (0x0000000DuL)
#define CAPSENSE_CSD_WAVEFORM_HSCMP_INV             (0x0000000EuL)
#define CAPSENSE_CSD_WAVEFORM_SENSE                 (0x0000000FuL)

/****************************************************************************
* HW CSD Block Register Mask Definition
****************************************************************************/

/* CSD_CONFIG register masks */
#define CAPSENSE_CSD_CONFIG_IREF_SEL_MSK                    (CSD_CONFIG_IREF_SEL_Msk)
#define CAPSENSE_CSD_CONFIG_FILTER_DELAY_2_CYCLES           (0x00000002Lu)
#define CAPSENSE_CSD_CONFIG_FILTER_DELAY_3_CYCLES           (0x00000003Lu)
#define CAPSENSE_CSD_CONFIG_FILTER_DELAY_4_CYCLES           (0x00000004Lu)
#define CAPSENSE_CSD_CONFIG_FILTER_DELAY_POS                (CSD_CONFIG_FILTER_DELAY_Pos)
#define CAPSENSE_CSD_CONFIG_FILTER_DELAY_MSK                (CSD_CONFIG_FILTER_DELAY_Msk)
#define CAPSENSE_CSD_CONFIG_FILTER_DELAY_12MHZ              (CAPSENSE_CSD_CONFIG_FILTER_DELAY_2_CYCLES << CSD_CONFIG_FILTER_DELAY_Pos)
#define CAPSENSE_CSD_CONFIG_FILTER_DELAY_24MHZ              (CAPSENSE_CSD_CONFIG_FILTER_DELAY_3_CYCLES << CSD_CONFIG_FILTER_DELAY_Pos)
#define CAPSENSE_CSD_CONFIG_FILTER_DELAY_48MHZ              (CAPSENSE_CSD_CONFIG_FILTER_DELAY_4_CYCLES << CSD_CONFIG_FILTER_DELAY_Pos)
#define CAPSENSE_CSD_CONFIG_SHIELD_DELAY_MSK                (CSD_CONFIG_SHIELD_DELAY_Msk)
#define CAPSENSE_CSD_CONFIG_SHIELD_DELAY_POS                (CSD_CONFIG_SHIELD_DELAY_Pos)
#define CAPSENSE_CSD_CONFIG_SENSE_EN_MSK                    (CSD_CONFIG_SENSE_EN_Msk)
#define CAPSENSE_CSD_CONFIG_DSI_COUNT_SEL_POS               (CSD_CONFIG_DSI_COUNT_SEL_Pos)
#define CAPSENSE_CSD_CONFIG_DSI_COUNT_SEL_MSK               (CSD_CONFIG_DSI_COUNT_SEL_Msk)
#define CAPSENSE_CSD_CONFIG_SAMPLE_SYNC_MSK                 (CSD_CONFIG_SAMPLE_SYNC_Msk)
#define CAPSENSE_CSD_CONFIG_ENABLE_MSK                      (CSD_CONFIG_ENABLE_Msk)

/* CSD_STAT_SEQ register masks */
#define CAPSENSE_CSD_STAT_SEQ_SEQ_STATE_MSK                 (CSD_STAT_SEQ_SEQ_STATE_Msk)
#define CAPSENSE_ADC_STAT_SEQ_SEQ_STATE_MSK                 (CSD_STAT_SEQ_ADC_STATE_Msk)

/* CSD_RESULT_VAL1 register masks */
#define CAPSENSE_CSD_RESULT_VAL1_VALUE_MSK                  (CSD_RESULT_VAL1_VALUE_Msk)
#define CAPSENSE_CSD_RESULT_VAL1_BAD_CONVS_MSK              (CSD_RESULT_VAL1_BAD_CONVS_Msk)
#define CAPSENSE_CSD_RESULT_VAL1_BAD_CONVS_POS              (CSD_RESULT_VAL1_BAD_CONVS_Pos)

/* CSD_RESULT_VAL2 register masks */
#define CAPSENSE_CSD_RESULT_VAL2_VALUE_MSK                  (CSD_RESULT_VAL2_VALUE_Msk)

/* CSD_INTR register masks */
#define CAPSENSE_CSD_INTR_SAMPLE_MSK                        (CSD_INTR_SAMPLE_Msk)
#define CAPSENSE_CSD_INTR_INIT_MSK                          (CSD_INTR_INIT_Msk)
#define CAPSENSE_CSD_INTR_ADC_RES_MSK                       (CSD_INTR_ADC_RES_Msk)
#define CAPSENSE_CSD_INTR_ALL_MSK                           (CSD_INTR_SAMPLE_Msk | CSD_INTR_INIT_Msk | CSD_INTR_ADC_RES_Msk)

/* CSD_INTR_MASK register masks */
#define CAPSENSE_CSD_INTR_MASK_SAMPLE_MSK                   (CSD_INTR_MASK_SAMPLE_Msk)
#define CAPSENSE_CSD_INTR_MASK_INIT_MSK                     (CSD_INTR_MASK_INIT_Msk)
#define CAPSENSE_CSD_INTR_MASK_ADC_RES_MSK                  (CSD_INTR_MASK_ADC_RES_Msk)
#define CAPSENSE_CSD_INTR_MASK_CLEAR_MSK                    (0uL)

/* CSD_HSCMP register masks */
#define CAPSENSE_CSD_HSCMP_HSCMP_EN_MSK                     (CSD_HSCMP_HSCMP_EN_Msk)
#define CAPSENSE_CSD_HSCMP_HSCMP_INVERT_MSK                 (CSD_HSCMP_HSCMP_INVERT_Msk)
#define CAPSENSE_CSD_HSCMP_AZ_EN_MSK                        (CSD_HSCMP_AZ_EN_Msk)

/* CSD_AMBUF register masks */
#define CAPSENSE_CSD_AMBUF_PWR_MODE_OFF                     (0uL)
#define CAPSENSE_CSD_AMBUF_PWR_MODE_NORM                    (1uL)
#define CAPSENSE_CSD_AMBUF_PWR_MODE_HI                      (2uL)

/* CSD_REFGEN register masks */
#define CAPSENSE_CSD_REFGEN_REFGEN_EN_MSK                   (CSD_REFGEN_REFGEN_EN_Msk)
#define CAPSENSE_CSD_REFGEN_BYPASS_MSK                      (CSD_REFGEN_BYPASS_Msk)
#define CAPSENSE_CSD_REFGEN_VDDA_EN_MSK                     (CSD_REFGEN_VDDA_EN_Msk)
#define CAPSENSE_CSD_REFGEN_RES_EN_MSK                      (CSD_REFGEN_RES_EN_Msk)
#define CAPSENSE_CSD_REFGEN_GAIN_POS                        (CSD_REFGEN_GAIN_Pos)
#define CAPSENSE_CSD_REFGEN_GAIN_MSK                        (CSD_REFGEN_GAIN_Msk)
#define CAPSENSE_CSD_REFGEN_VREFLO_SEL_MSK                  (CSD_REFGEN_VREFLO_SEL_Msk)
#define CAPSENSE_CSD_REFGEN_VREFLO_INT_MSK                  (CSD_REFGEN_VREFLO_INT_Msk)

#define CAPSENSE_VREF_HI_OVERSHOOT_CORRECTION               (0x00000001uL)

#if(CAPSENSE_ENABLE != CAPSENSE_LP_MODE_EN)
    #define CAPSENSE_DEFAULT_CSD_REFGEN_RES_CFG             (CAPSENSE_CSD_REFGEN_RES_EN_MSK)
#else
    #define CAPSENSE_DEFAULT_CSD_REFGEN_RES_CFG             (0u)
#endif /* (CAPSENSE_ENABLE != CAPSENSE_LP_MODE_EN) */

#define CAPSENSE_DEFAULT_CSD_REFGEN_GAIN                    (CAPSENSE_CSD_GAIN << CSD_REFGEN_GAIN_Pos)
#define CAPSENSE_DEFAULT_CSD_REFGEN_VREFLO_SEL              (((CSD_REFGEN_VREFLO_SEL_Msk >> CSD_REFGEN_VREFLO_SEL_Pos) - CAPSENSE_VREF_HI_OVERSHOOT_CORRECTION) <<  CSD_REFGEN_VREFLO_SEL_Pos)
#define CAPSENSE_CSD_REFGEN_NO_SHIELD_CFG                   (CAPSENSE_CSD_REFGEN_REFGEN_EN_MSK | CAPSENSE_DEFAULT_CSD_REFGEN_RES_CFG | CAPSENSE_DEFAULT_CSD_REFGEN_GAIN)
#define CAPSENSE_CSD_REFGEN_WITH_SHIELD_CFG                 (CAPSENSE_CSD_REFGEN_REFGEN_EN_MSK | CAPSENSE_CSD_REFGEN_RES_EN_MSK | CAPSENSE_DEFAULT_CSD_REFGEN_GAIN)

/* CSD_CSDCMP register masks */
#define CAPSENSE_CSD_CSDCMP_CSDCMP_DISABLED                 (0uL)
#define CAPSENSE_CSD_CSDCMP_CSDCMP_EN_MSK                   (CSD_CSDCMP_CSDCMP_EN_Msk)
#define CAPSENSE_CSD_CSDCMP_CMP_PHASE_FULL                  (0uL << CSD_CSDCMP_CMP_PHASE_Pos)
#define CAPSENSE_CSD_CSDCMP_CMP_PHASE_PHI1                  (1uL << CSD_CSDCMP_CMP_PHASE_Pos)
#define CAPSENSE_CSD_CSDCMP_CMP_PHASE_PHI2                  (2uL << CSD_CSDCMP_CMP_PHASE_Pos)
#define CAPSENSE_CSD_CSDCMP_CMP_PHASE_PHI1_2                (3uL << CSD_CSDCMP_CMP_PHASE_Pos)
#define CAPSENSE_CSD_CSDCMP_AZ_EN_MSK                       (CSD_CSDCMP_AZ_EN_Msk)

/* CSD_SENSE_PERIOD register masks */
#define CAPSENSE_CSD_SENSE_PERIOD_SENSE_DIV_MSK             (CSD_SENSE_PERIOD_SENSE_DIV_Msk)
#define CAPSENSE_CSD_SENSE_PERIOD_LFSR_SIZE_MSK             (CSD_SENSE_PERIOD_LFSR_SIZE_Msk)
#define CAPSENSE_CSD_SENSE_PERIOD_LFSR_SIZE_POS             (CSD_SENSE_PERIOD_LFSR_SIZE_Pos)
#define CAPSENSE_CSD_SENSE_PERIOD_LFSR_SCALE_MSK            (CSD_SENSE_PERIOD_LFSR_SCALE_Msk)
#define CAPSENSE_CSD_SENSE_PERIOD_LFSR_CLEAR_MSK            (CSD_SENSE_PERIOD_LFSR_CLEAR_Msk)
#define CAPSENSE_CSD_SENSE_PERIOD_SEL_LFSR_MSB_MSK          (CSD_SENSE_PERIOD_SEL_LFSR_MSB_Msk)
#define CAPSENSE_CSD_SENSE_PERIOD_LFSR_BITS_MSK             (CSD_SENSE_PERIOD_LFSR_BITS_Msk)

/* CSD_SENSE_DUTY register masks */
#define CAPSENSE_CSD_SENSE_DUTY_SENSE_WIDTH_MSK             (CSD_SENSE_DUTY_SENSE_WIDTH_Msk)
#define CAPSENSE_CSD_SENSE_DUTY_SENSE_POL_MSK               (CSD_SENSE_DUTY_SENSE_POL_Msk)
#define CAPSENSE_CSD_SENSE_DUTY_SENSE_POL_PHI_LOW           (0uL)
#define CAPSENSE_CSD_SENSE_DUTY_SENSE_POL_PHI_HIGH          (CSD_SENSE_DUTY_SENSE_POL_Msk)
#define CAPSENSE_CSD_SENSE_DUTY_OVERLAP_PHI1_MSK            (CSD_SENSE_DUTY_OVERLAP_PHI1_Msk)
#define CAPSENSE_CSD_SENSE_DUTY_OVERLAP_PHI2_MSK            (CSD_SENSE_DUTY_OVERLAP_PHI2_Msk)

#define CAPSENSE_CSD_SW_HS_P_SEL_SW_HMPM_STATIC_OPEN        (CAPSENSE_CSD_WAVEFORM_STATIC_OPEN << CSD_SW_HS_P_SEL_SW_HMPM_Pos)
#define CAPSENSE_CSD_SW_HS_P_SEL_SW_HMPM_STATIC_CLOSE       (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_HS_P_SEL_SW_HMPM_Pos)
#define CAPSENSE_CSD_SW_HS_P_SEL_SW_HMPT_STATIC_OPEN        (CAPSENSE_CSD_WAVEFORM_STATIC_OPEN << CSD_SW_HS_P_SEL_SW_HMPT_Pos)
#define CAPSENSE_CSD_SW_HS_P_SEL_SW_HMPT_STATIC_CLOSE       (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_HS_P_SEL_SW_HMPT_Pos)
#define CAPSENSE_CSD_SW_HS_P_SEL_SW_HMPS_STATIC_OPEN        (CAPSENSE_CSD_WAVEFORM_STATIC_OPEN << CSD_SW_HS_P_SEL_SW_HMPS_Pos)
#define CAPSENSE_CSD_SW_HS_P_SEL_SW_HMPS_STATIC_CLOSE       (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_HS_P_SEL_SW_HMPS_Pos)
#define CAPSENSE_CSD_SW_HS_P_SEL_SW_HMMA_STATIC_OPEN        (CAPSENSE_CSD_WAVEFORM_STATIC_OPEN << CSD_SW_HS_P_SEL_SW_HMMA_Pos)
#define CAPSENSE_CSD_SW_HS_P_SEL_SW_HMMA_STATIC_CLOSE       (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_HS_P_SEL_SW_HMMA_Pos)
#define CAPSENSE_CSD_SW_HS_P_SEL_SW_HMMB_STATIC_OPEN        (CAPSENSE_CSD_WAVEFORM_STATIC_OPEN << CSD_SW_HS_P_SEL_SW_HMMB_Pos)
#define CAPSENSE_CSD_SW_HS_P_SEL_SW_HMMB_STATIC_CLOSE       (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_HS_P_SEL_SW_HMMB_Pos)
#define CAPSENSE_CSD_SW_HS_P_SEL_SW_HMRH_STATIC_OPEN        (CAPSENSE_CSD_WAVEFORM_STATIC_OPEN << CSD_SW_HS_P_SEL_SW_HMRH_Pos)
#define CAPSENSE_CSD_SW_HS_P_SEL_SW_HMRH_STATIC_CLOSE       (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_HS_P_SEL_SW_HMRH_Pos)

#define CAPSENSE_CSD_SW_HS_N_SEL_SW_HCRH_STATIC_OPEN        (CAPSENSE_CSD_WAVEFORM_STATIC_OPEN << CSD_SW_HS_N_SEL_SW_HCRH_Pos)
#define CAPSENSE_CSD_SW_HS_N_SEL_SW_HCRH_STATIC_CLOSE       (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_HS_N_SEL_SW_HCRH_Pos)
#define CAPSENSE_CSD_SW_HS_N_SEL_SW_HCRL_STATIC_OPEN        (CAPSENSE_CSD_WAVEFORM_STATIC_OPEN << CSD_SW_HS_N_SEL_SW_HCRL_Pos)
#define CAPSENSE_CSD_SW_HS_N_SEL_SW_HCRL_STATIC_CLOSE       (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_HS_N_SEL_SW_HCRL_Pos)
#define CAPSENSE_CSD_SW_HS_N_SEL_SW_HCCD_STATIC_OPEN        (CAPSENSE_CSD_WAVEFORM_STATIC_OPEN << CSD_SW_HS_N_SEL_SW_HCCD_Pos)
#define CAPSENSE_CSD_SW_HS_N_SEL_SW_HCCD_STATIC_CLOSE       (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_HS_N_SEL_SW_HCCD_Pos)

#define CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCAV_HSCMP            (CAPSENSE_CSD_WAVEFORM_HSCMP << CSD_SW_SHIELD_SEL_SW_HCAV_Pos)
#define CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCAV_PHI1             (CAPSENSE_CSD_WAVEFORM_PHI1 << CSD_SW_SHIELD_SEL_SW_HCAV_Pos)
#define CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCAV_PHI2             (CAPSENSE_CSD_WAVEFORM_PHI2 << CSD_SW_SHIELD_SEL_SW_HCAV_Pos)
#define CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCAV_PHI1_HSCMP       (CAPSENSE_CSD_WAVEFORM_PHI1_HSCMP << CSD_SW_SHIELD_SEL_SW_HCAV_Pos)
#define CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCAV_PHI2_HSCMP       (CAPSENSE_CSD_WAVEFORM_PHI2_HSCMP << CSD_SW_SHIELD_SEL_SW_HCAV_Pos)
#define CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCAV_STATIC_OPEN      (CAPSENSE_CSD_WAVEFORM_STATIC_OPEN << CSD_SW_SHIELD_SEL_SW_HCAV_Pos)
#define CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCAV_STATIC_CLOSE     (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_SHIELD_SEL_SW_HCAV_Pos)

#define CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCAG_HSCMP            (CAPSENSE_CSD_WAVEFORM_HSCMP << CSD_SW_SHIELD_SEL_SW_HCAG_Pos)
#define CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCAG_PHI1             (CAPSENSE_CSD_WAVEFORM_PHI1 << CSD_SW_SHIELD_SEL_SW_HCAG_Pos)
#define CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCAG_PHI2             (CAPSENSE_CSD_WAVEFORM_PHI2 << CSD_SW_SHIELD_SEL_SW_HCAG_Pos)
#define CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCAG_PHI1_HSCMP       (CAPSENSE_CSD_WAVEFORM_PHI1_HSCMP << CSD_SW_SHIELD_SEL_SW_HCAG_Pos)
#define CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCAG_PHI2_HSCMP       (CAPSENSE_CSD_WAVEFORM_PHI2_HSCMP << CSD_SW_SHIELD_SEL_SW_HCAG_Pos)
#define CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCAG_STATIC_OPEN      (CAPSENSE_CSD_WAVEFORM_STATIC_OPEN << CSD_SW_SHIELD_SEL_SW_HCAG_Pos)
#define CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCAG_STATIC_CLOSE     (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_SHIELD_SEL_SW_HCAG_Pos)

#define CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCBV_HSCMP            (CAPSENSE_CSD_WAVEFORM_HSCMP << CSD_SW_SHIELD_SEL_SW_HCBV_Pos)
#define CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCBV_PHI1             (CAPSENSE_CSD_WAVEFORM_PHI1 << CSD_SW_SHIELD_SEL_SW_HCBV_Pos)
#define CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCBV_PHI2             (CAPSENSE_CSD_WAVEFORM_PHI2 << CSD_SW_SHIELD_SEL_SW_HCBV_Pos)
#define CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCBV_PHI1_HSCMP       (CAPSENSE_CSD_WAVEFORM_PHI1_HSCMP << CSD_SW_SHIELD_SEL_SW_HCBV_Pos)
#define CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCBV_PHI2_HSCMP       (CAPSENSE_CSD_WAVEFORM_PHI2_HSCMP << CSD_SW_SHIELD_SEL_SW_HCBV_Pos)
#define CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCBV_STATIC_OPEN      (CAPSENSE_CSD_WAVEFORM_STATIC_OPEN << CSD_SW_SHIELD_SEL_SW_HCBV_Pos)
#define CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCBV_STATIC_CLOSE      (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_SHIELD_SEL_SW_HCBV_Pos)

#define CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCBG_HSCMP            (CAPSENSE_CSD_WAVEFORM_HSCMP << CSD_SW_SHIELD_SEL_SW_HCBG_Pos)
#define CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCBG_PHI1             (CAPSENSE_CSD_WAVEFORM_PHI1 << CSD_SW_SHIELD_SEL_SW_HCBG_Pos)
#define CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCBG_PHI2             (CAPSENSE_CSD_WAVEFORM_PHI2 << CSD_SW_SHIELD_SEL_SW_HCBG_Pos)
#define CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCBG_PHI1_HSCMP       (CAPSENSE_CSD_WAVEFORM_PHI1_HSCMP << CSD_SW_SHIELD_SEL_SW_HCBG_Pos)
#define CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCBG_PHI2_HSCMP       (CAPSENSE_CSD_WAVEFORM_PHI2_HSCMP << CSD_SW_SHIELD_SEL_SW_HCBG_Pos)
#define CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCBG_STATIC_OPEN      (CAPSENSE_CSD_WAVEFORM_STATIC_OPEN << CSD_SW_SHIELD_SEL_SW_HCBG_Pos)
#define CAPSENSE_CSD_SW_SHIELD_SEL_SW_HCBG_STATIC_CLOSE      (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_SHIELD_SEL_SW_HCBG_Pos)

#define CAPSENSE_CSD_SW_AMUXBUF_SEL_SW_IRLB_STATIC_CLOSE    (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_AMUXBUF_SEL_SW_IRLB_Pos)
#define CAPSENSE_CSD_SW_AMUXBUF_SEL_SW_IRH_STATIC_CLOSE     (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_AMUXBUF_SEL_SW_IRH_Pos)
#define CAPSENSE_CSD_SW_AMUXBUF_SEL_SW_ICB_PHI2             (CAPSENSE_CSD_WAVEFORM_PHI2 << CSD_SW_AMUXBUF_SEL_SW_ICB_Pos)
#define CAPSENSE_CSD_SW_AMUXBUF_SEL_SW_DEFAULT              (CAPSENSE_CSD_WAVEFORM_STATIC_OPEN)

/* CSD_SW_BYP_SEL register masks */
#define CAPSENSE_CSD_SW_BYP_SEL_SW_BYA_MSK                  (CSD_SW_BYP_SEL_SW_BYA_Msk)
#define CAPSENSE_CSD_SW_BYP_SEL_SW_BYB_MSK                  (CSD_SW_BYP_SEL_SW_BYB_Msk)

#define CAPSENSE_CSD_SW_CMP_P_SEL_SW_SFPM_STATIC_OPEN       (CAPSENSE_CSD_WAVEFORM_STATIC_OPEN << CSD_SW_CMP_P_SEL_SW_SFPM_Pos)
#define CAPSENSE_CSD_SW_CMP_P_SEL_SW_SFPM_STATIC_CLOSE      (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_CMP_P_SEL_SW_SFPM_Pos)
#define CAPSENSE_CSD_SW_CMP_P_SEL_SW_SFPT_STATIC_OPEN       (CAPSENSE_CSD_WAVEFORM_STATIC_OPEN << CSD_SW_CMP_P_SEL_SW_SFPT_Pos)
#define CAPSENSE_CSD_SW_CMP_P_SEL_SW_SFPT_STATIC_CLOSE      (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_CMP_P_SEL_SW_SFPT_Pos)
#define CAPSENSE_CSD_SW_CMP_P_SEL_SW_SFPS_STATIC_OPEN       (CAPSENSE_CSD_WAVEFORM_STATIC_OPEN << CSD_SW_CMP_P_SEL_SW_SFPS_Pos)
#define CAPSENSE_CSD_SW_CMP_P_SEL_SW_SFPS_STATIC_CLOSE      (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_CMP_P_SEL_SW_SFPS_Pos)
#define CAPSENSE_CSD_SW_CMP_P_SEL_SW_SFMA_STATIC_OPEN       (CAPSENSE_CSD_WAVEFORM_STATIC_OPEN << CSD_SW_CMP_P_SEL_SW_SFMA_Pos)
#define CAPSENSE_CSD_SW_CMP_P_SEL_SW_SFMA_STATIC_CLOSE      (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_CMP_P_SEL_SW_SFMA_Pos)
#define CAPSENSE_CSD_SW_CMP_P_SEL_SW_SFCA_STATIC_OPEN       (CAPSENSE_CSD_WAVEFORM_STATIC_OPEN << CSD_SW_CMP_P_SEL_SW_SFCA_Pos)
#define CAPSENSE_CSD_SW_CMP_P_SEL_SW_SFCA_STATIC_CLOSE      (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_CMP_P_SEL_SW_SFCA_Pos)
#define CAPSENSE_CSD_SW_CMP_P_SEL_SW_SFCA_MSK               (CSD_SW_CMP_P_SEL_SW_SFCA_Msk)

#define CAPSENSE_CSD_SW_CMP_N_SEL_SW_SCRH_STATIC_OPEN       (CAPSENSE_CSD_WAVEFORM_STATIC_OPEN << CSD_SW_CMP_N_SEL_SW_SCRH_Pos)
#define CAPSENSE_CSD_SW_CMP_N_SEL_SW_SCRH_STATIC_CLOSE      (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_CMP_N_SEL_SW_SCRH_Pos)
#define CAPSENSE_CSD_SW_CMP_N_SEL_SW_SCRL_STATIC_OPEN       (CAPSENSE_CSD_WAVEFORM_STATIC_OPEN << CSD_SW_CMP_N_SEL_SW_SCRL_Pos)
#define CAPSENSE_CSD_SW_CMP_N_SEL_SW_SCRL_STATIC_CLOSE      (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_CMP_N_SEL_SW_SCRL_Pos)

/* CSD_SW_REFGEN_SEL register masks */
#define CAPSENSE_CSD_SW_REFGEN_SEL_SW_IAIB_MSK              (CSD_SW_REFGEN_SEL_SW_IAIB_Msk)
#define CAPSENSE_CSD_SW_REFGEN_SEL_SW_IBCB_MSK              (CSD_SW_REFGEN_SEL_SW_IBCB_Msk)
#define CAPSENSE_CSD_SW_REFGEN_SEL_SW_SGMB_MSK              (CSD_SW_REFGEN_SEL_SW_SGMB_Msk)
#define CAPSENSE_CSD_SW_REFGEN_SEL_SW_SGRP_MSK              (CSD_SW_REFGEN_SEL_SW_SGRP_Msk)
#define CAPSENSE_CSD_SW_REFGEN_SEL_SW_SGRE_MSK              (CSD_SW_REFGEN_SEL_SW_SGRE_Msk)
#define CAPSENSE_CSD_SW_REFGEN_SEL_SW_SGR_MSK               (CSD_SW_REFGEN_SEL_SW_SGR_Msk)

#define CAPSENSE_CSD_SW_FW_MOD_SEL_SW_F1PM_MSK              (CSD_SW_FW_MOD_SEL_SW_F1PM_Msk)
#define CAPSENSE_CSD_SW_FW_MOD_SEL_SW_F1PM_STATIC_CLOSE     (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_FW_MOD_SEL_SW_F1PM_Pos)
#define CAPSENSE_CSD_SW_FW_MOD_SEL_SW_F1MA_MSK              (CSD_SW_FW_MOD_SEL_SW_F1MA_Msk)
#define CAPSENSE_CSD_SW_FW_MOD_SEL_SW_F1MA_STATIC_CLOSE     (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_FW_MOD_SEL_SW_F1MA_Pos)
#define CAPSENSE_CSD_SW_FW_MOD_SEL_SW_F1CA_MSK              (CSD_SW_FW_MOD_SEL_SW_F1CA_Msk)
#define CAPSENSE_CSD_SW_FW_MOD_SEL_SW_F1CA_STATIC_CLOSE     (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_FW_MOD_SEL_SW_F1CA_Pos)
#define CAPSENSE_CSD_SW_FW_MOD_SEL_SW_F1CA_PHI2             (CAPSENSE_CSD_WAVEFORM_PHI2 << CSD_SW_FW_MOD_SEL_SW_F1CA_Pos)
#define CAPSENSE_CSD_SW_FW_MOD_SEL_SW_C1CC_STATIC_OPEN      (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_FW_MOD_SEL_SW_C1CC_Pos)
#define CAPSENSE_CSD_SW_FW_MOD_SEL_SW_C1CC_STATIC_CLOSE     (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_FW_MOD_SEL_SW_C1CC_Pos)
#define CAPSENSE_CSD_SW_FW_MOD_SEL_SW_C1CD_STATIC_OPEN      (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_FW_MOD_SEL_SW_C1CD_Pos)
#define CAPSENSE_CSD_SW_FW_MOD_SEL_SW_C1CD_STATIC_CLOSE     (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_FW_MOD_SEL_SW_C1CD_Pos)

#define CAPSENSE_CSD_SW_FW_TANK_SEL_SW_F2PT_MSK             (CSD_SW_FW_TANK_SEL_SW_F2PT_Msk)
#define CAPSENSE_CSD_SW_FW_TANK_SEL_SW_F2PT_STATIC_CLOSE    (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_FW_TANK_SEL_SW_F2PT_Pos)
#define CAPSENSE_CSD_SW_FW_TANK_SEL_SW_F2MA_MSK             (CSD_SW_FW_TANK_SEL_SW_F2MA_Msk)
#define CAPSENSE_CSD_SW_FW_TANK_SEL_SW_F2MA_STATIC_CLOSE    (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_FW_TANK_SEL_SW_F2MA_Pos)
#define CAPSENSE_CSD_SW_FW_TANK_SEL_SW_F2CA_MSK             (CSD_SW_FW_TANK_SEL_SW_F2CA_Msk)
#define CAPSENSE_CSD_SW_FW_TANK_SEL_SW_F2CA_STATIC_CLOSE    (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_FW_TANK_SEL_SW_F2CA_Pos)
#define CAPSENSE_CSD_SW_FW_TANK_SEL_SW_F2CB_MSK             (CSD_SW_FW_TANK_SEL_SW_F2CB_Msk)
#define CAPSENSE_CSD_SW_FW_TANK_SEL_SW_F2CB_STATIC_CLOSE    (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_FW_TANK_SEL_SW_F2CB_Pos)
#define CAPSENSE_CSD_SW_FW_TANK_SEL_SW_F2CA_PHI2            (CAPSENSE_CSD_WAVEFORM_PHI2 << CSD_SW_FW_TANK_SEL_SW_F2CA_Pos)
#define CAPSENSE_CSD_SW_FW_TANK_SEL_SW_F2CB_PHI2            (CAPSENSE_CSD_WAVEFORM_PHI2 << CSD_SW_FW_TANK_SEL_SW_F2CB_Pos)
#define CAPSENSE_CSD_SW_FW_TANK_SEL_SW_C2CC_STATIC_OPEN     (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_FW_TANK_SEL_SW_C2CC_Pos)
#define CAPSENSE_CSD_SW_FW_TANK_SEL_SW_C2CC_STATIC_CLOSE    (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_FW_TANK_SEL_SW_C2CC_Pos)
#define CAPSENSE_CSD_SW_FW_TANK_SEL_SW_C2CD_STATIC_OPEN     (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_FW_TANK_SEL_SW_C2CD_Pos)
#define CAPSENSE_CSD_SW_FW_TANK_SEL_SW_C2CD_STATIC_CLOSE    (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_SW_FW_TANK_SEL_SW_C2CD_Pos)

#define CAPSENSE_CSD_TX_OUT_MSK                             (CSD_IO_SEL_CSD_TX_OUT_Msk)
#define CAPSENSE_CSD_TX_OUT_STATIC_OPEN                     (CAPSENSE_CSD_WAVEFORM_STATIC_OPEN << CSD_IO_SEL_CSD_TX_OUT_Pos)

#define CAPSENSE_CSD_TX_OUT_EN_MSK                          (CSD_IO_SEL_CSD_TX_OUT_EN_Msk)
#define CAPSENSE_CSD_TX_OUT_EN_PHI1_DELAY                   (CAPSENSE_CSD_WAVEFORM_PHI1_DELAY << 4u)
#define CAPSENSE_CSD_TX_AMUXB_EN_MSK                        (CSD_IO_SEL_CSD_TX_AMUXB_EN_Msk)
#define CAPSENSE_CSD_TX_AMUXB_EN_PHI2_DELAY                 (CAPSENSE_CSD_WAVEFORM_PHI2_DELAY << 12u)
#define CAPSENSE_CSD_TX_N_OUT_MSK                           (CSD_IO_SEL_CSD_TX_N_OUT_Msk)
#define CAPSENSE_CSD_TX_N_OUT_STATIC_OPEN                   (CAPSENSE_CSD_WAVEFORM_STATIC_OPEN << CSD_IO_SEL_CSD_TX_N_OUT_Pos)
#define CAPSENSE_CSD_TX_N_OUT_STATIC_CLOSE                  (CAPSENSE_CSD_WAVEFORM_STATIC_CLOSED << CSD_IO_SEL_CSD_TX_N_OUT_Pos)
#define CAPSENSE_CSD_TX_N_OUT_EN_MSK                        (CSD_IO_SEL_CSD_TX_N_OUT_EN_Msk)
#define CAPSENSE_CSD_TX_N_OUT_EN_PHI1                       (CAPSENSE_CSD_WAVEFORM_PHI1 << CSD_IO_SEL_CSD_TX_N_OUT_EN_Pos)
#define CAPSENSE_CSD_TX_N_AMUXA_EN_MSK                      (CSD_IO_SEL_CSD_TX_N_AMUXA_EN_Msk)
#define CAPSENSE_CSD_TX_N_AMUXA_EN_PHI2                     (CAPSENSE_CSD_WAVEFORM_PHI2 << CSD_IO_SEL_CSD_TX_N_AMUXA_EN_Pos)

/* IDACB used */
#if ((CAPSENSE_ENABLE == CAPSENSE_CSD_IDAC_COMP_EN) && \
     (CAPSENSE_ENABLE == CAPSENSE_CSD_DEDICATED_IDAC_COMP_EN))
        #define CAPSENSE_IDACB_USED                         (1u)
#else
        #define CAPSENSE_IDACB_USED                         (0u)
#endif /* ((CAPSENSE_ENABLE == CAPSENSE_CSD_IDAC_COMP_EN) && \
           (CAPSENSE_ENABLE == CAPSENSE_CSD_DEDICATED_IDAC_COMP_EN)) */

/* CSD_SEQ_INIT_CNT register masks */
#define CAPSENSE_CSD_SEQ_INIT_CNT_CONV_CNT_MSK              (CSD_SEQ_INIT_CNT_CONV_CNT_Msk)

/* CSD_SEQ_NORM_CNT register masks */
#define CAPSENSE_CSD_SEQ_NORM_CNT_CONV_CNT_MSK              (CSD_SEQ_NORM_CNT_CONV_CNT_Msk)

/* CSD_SEQ_START register masks */
#define CAPSENSE_CSD_SEQ_START_START_MSK                    (CSD_SEQ_START_START_Msk)
#define CAPSENSE_CSD_SEQ_START_SEQ_MODE_MSK                 (CSD_SEQ_START_SEQ_MODE_Msk)
#define CAPSENSE_CSD_SEQ_START_ABORT_MSK                    (CSD_SEQ_START_ABORT_Msk)
#define CAPSENSE_CSD_SEQ_START_DSI_START_EN_MSK             (CSD_SEQ_START_DSI_START_EN_Msk)
#define CAPSENSE_CSD_SEQ_START_AZ0_SKIP_MSK                 (CSD_SEQ_START_AZ0_SKIP_Msk)
#define CAPSENSE_CSD_SEQ_START_AZ1_SKIP_MSK                 (CSD_SEQ_START_AZ1_SKIP_Msk)

/* CSD_IDACA register masks */
#define CAPSENSE_CSD_IDACA_VAL_MSK                          (CSD_IDACA_VAL_Msk)
#define CAPSENSE_CSD_IDACA_VAL_POS                          (CSD_IDACA_VAL_Pos)

#define CAPSENSE_CSD_IDACA_POL_DYN_MSK                      (CSD_IDACA_POL_DYN_Msk)
#define CAPSENSE_CSD_IDACA_POL_DYN_POS                      (CSD_IDACA_POL_DYN_Pos)

#define CAPSENSE_CSD_IDACA_POLARITY_MSK                     (CSD_IDACA_POLARITY_Msk)
#define CAPSENSE_CSD_IDACA_POLARITY_POS                     (CSD_IDACA_POLARITY_Pos)
#define CAPSENSE_CSD_IDACA_POLARITY_VSSA_SRC                (0uL)
#define CAPSENSE_CSD_IDACA_POLARITY_VDDA_SNK                (1uL)
#define CAPSENSE_CSD_IDACA_POLARITY_SENSE                   (2uL)
#define CAPSENSE_CSD_IDACA_POLARITY_SENSE_INV               (3uL)

#define CAPSENSE_CSD_IDACA_BALL_MODE_MSK                    (CSD_IDACA_BAL_MODE_Msk)
#define CAPSENSE_CSD_IDACA_BALL_MODE_POS                    (CSD_IDACA_BAL_MODE_Pos)
#define CAPSENSE_CSD_IDACA_BALL_MODE_FULL                   (0uL)
#define CAPSENSE_CSD_IDACA_BALL_MODE_PHI1                   (1uL)
#define CAPSENSE_CSD_IDACA_BALL_MODE_PHI2                   (2uL)
#define CAPSENSE_CSD_IDACA_BALL_MODE_PHI1_2                 (3uL)

#define CAPSENSE_CSD_IDACA_LEG1_MODE_MSK                    (CSD_IDACA_LEG1_MODE_Msk)
#define CAPSENSE_CSD_IDACA_LEG1_MODE_POS                    (CSD_IDACA_LEG1_MODE_Pos)
#define CAPSENSE_CSD_IDACA_LEG1_MODE_GP_STATIC              (0uL)
#define CAPSENSE_CSD_IDACA_LEG1_MODE_GP                     (1uL)
#define CAPSENSE_CSD_IDACA_LEG1_MODE_CSD_STATIC             (2uL)
#define CAPSENSE_CSD_IDACA_LEG1_MODE_CSD                    (3uL)

#define CAPSENSE_CSD_IDACA_LEG2_MODE_MSK                    (CSD_IDACA_LEG2_MODE_Msk)
#define CAPSENSE_CSD_IDACA_LEG2_MODE_POS                    (CSD_IDACA_LEG2_MODE_Pos)
#define CAPSENSE_CSD_IDACA_LEG2_MODE_GP_STATIC              (0uL)
#define CAPSENSE_CSD_IDACA_LEG2_MODE_GP                     (1uL)
#define CAPSENSE_CSD_IDACA_LEG2_MODE_CSD_STATIC             (2uL)
#define CAPSENSE_CSD_IDACA_LEG2_MODE_CSD                    (3uL)

#define CAPSENSE_CSD_IDACA_RANGE_MSK                        (CSD_IDACA_RANGE_Msk)
#define CAPSENSE_CSD_IDACA_RANGE_POS                        (CSD_IDACA_RANGE_Pos)
#define CAPSENSE_CSD_IDACA_RANGE_IDAC_LO                    (0uL)
#define CAPSENSE_CSD_IDACA_RANGE_IDAC_MED                   (1uL)
#define CAPSENSE_CSD_IDACA_RANGE_IDAC_HI                    (2uL)

#define CAPSENSE_CSD_IDACA_LEG1_EN_MSK                      (CSD_IDACA_LEG1_EN_Msk)
#define CAPSENSE_CSD_IDACA_LEG2_EN_MSK                      (CSD_IDACA_LEG2_EN_Msk)

/* CSD_IDACB register masks */
#define CAPSENSE_CSD_IDACB_VAL_MSK                          (CSD_IDACB_VAL_Msk)
#define CAPSENSE_CSD_IDACB_VAL_POS                          (CSD_IDACB_VAL_Pos)

#define CAPSENSE_CSD_IDACB_POL_DYN_MSK                      (CSD_IDACB_POL_DYN_Msk)
#define CAPSENSE_CSD_IDACB_POL_DYN_POS                      (CSD_IDACB_POL_DYN_Pos)
#define CAPSENSE_CSD_IDACB_POL_DYN_STATIC                   (0uL)
#define CAPSENSE_CSD_IDACB_POL_DYN_DYNAMIC                  (1uL)

#define CAPSENSE_CSD_IDACB_POLARITY_MSK                     (CSD_IDACB_POLARITY_Msk)
#define CAPSENSE_CSD_IDACB_POLARITY_POS                     (CSD_IDACB_POLARITY_Pos)
#define CAPSENSE_CSD_IDACB_POLARITY_VSSA_SRC                (0uL)
#define CAPSENSE_CSD_IDACB_POLARITY_VDDA_SNK                (1uL)
#define CAPSENSE_CSD_IDACB_POLARITY_SENSE                   (2uL)
#define CAPSENSE_CSD_IDACB_POLARITY_SENSE_INV               (3uL)

#define CAPSENSE_CSD_IDACB_BALL_MODE_MSK                    (CSD_IDACB_BAL_MODE_Msk)
#define CAPSENSE_CSD_IDACB_BALL_MODE_POS                    (CSD_IDACB_BAL_MODE_Pos)
#define CAPSENSE_CSD_IDACB_BALL_MODE_FULL                   (0uL)
#define CAPSENSE_CSD_IDACB_BALL_MODE_PHI1                   (1uL)
#define CAPSENSE_CSD_IDACB_BALL_MODE_PHI2                   (2uL)
#define CAPSENSE_CSD_IDACB_BALL_MODE_PHI1_2                 (3uL)

#define CAPSENSE_CSD_IDACB_LEG1_MODE_MSK                    (CSD_IDACB_LEG1_MODE_Msk)
#define CAPSENSE_CSD_IDACB_LEG1_MODE_POS                    (CSD_IDACB_LEG1_MODE_Pos)
#define CAPSENSE_CSD_IDACB_LEG1_MODE_GP_STATIC              (0uL)
#define CAPSENSE_CSD_IDACB_LEG1_MODE_GP                     (1uL)
#define CAPSENSE_CSD_IDACB_LEG1_MODE_CSD_STATIC             (2uL)
#define CAPSENSE_CSD_IDACB_LEG1_MODE_CSD                    (3uL)

#define CAPSENSE_CSD_IDACB_LEG2_MODE_MSK                    (CSD_IDACB_LEG2_MODE_Msk)
#define CAPSENSE_CSD_IDACB_LEG2_MODE_POS                    (CSD_IDACB_LEG2_MODE_Pos)
#define CAPSENSE_CSD_IDACB_LEG2_MODE_GP_STATIC              (0uL)
#define CAPSENSE_CSD_IDACB_LEG2_MODE_GP                     (1uL)
#define CAPSENSE_CSD_IDACB_LEG2_MODE_CSD_STATIC             (2uL)
#define CAPSENSE_CSD_IDACB_LEG2_MODE_CSD                    (3uL)

#define CAPSENSE_CSD_IDACB_RANGE_MSK                        (CSD_IDACB_RANGE_Msk)
#define CAPSENSE_CSD_IDACB_RANGE_POS                        (CSD_IDACB_RANGE_Pos)
#define CAPSENSE_CSD_IDACB_RANGE_IDAC_LO                    (0uL)
#define CAPSENSE_CSD_IDACB_RANGE_IDAC_MED                   (1uL)
#define CAPSENSE_CSD_IDACB_RANGE_IDAC_HI                    (2uL)

#define CAPSENSE_CSD_IDACB_LEG1_EN_MSK                      (CSD_IDACB_LEG1_EN_Msk)
#define CAPSENSE_CSD_IDACB_LEG2_EN_MSK                      (CSD_IDACB_LEG2_EN_Msk)
#define CAPSENSE_CSD_IDACB_LEG3_EN_MSK                      (CSD_IDACB_LEG3_EN_Msk)

/***************************************
* PRS & LFSR masks
***************************************/
#define CAPSENSE_LFSR_TABLE_SIZE                            (4u)

#define CAPSENSE_PRS_LENGTH_2_BITS                          (0x00000003Lu)
#define CAPSENSE_PRS_LENGTH_3_BITS                          (0x00000007Lu)
#define CAPSENSE_PRS_LENGTH_4_BITS                          (0x0000000FLu)
#define CAPSENSE_PRS_LENGTH_5_BITS                          (0x0000001FLu)
#define CAPSENSE_PRS_LENGTH_8_BITS                          (0x000000FFLu)
#define CAPSENSE_PRS_LENGTH_12_BITS                         (0x00000FFFLu)

#define CAPSENSE_SNSCLK_SSC1_PERIOD                         (63u)
#define CAPSENSE_SNSCLK_SSC2_PERIOD                         (127u)
#define CAPSENSE_SNSCLK_SSC3_PERIOD                         (511u)
#define CAPSENSE_SNSCLK_SSC4_PERIOD                         (1023u)

#define CAPSENSE_SNSCLK_SSC1_RANGE                          (16u)
#define CAPSENSE_SNSCLK_SSC2_RANGE                          (16u)
#define CAPSENSE_SNSCLK_SSC3_RANGE                          (16u)
#define CAPSENSE_SNSCLK_SSC4_RANGE                          (16u)

#define CAPSENSE_LFSR_DITHER_PERCENTAGE                     (10uL)
#define CAPSENSE_SNSCLK_SSC1_THRESHOLD                      (CAPSENSE_LFSR_DITHER_PERCENTAGE * CAPSENSE_SNSCLK_SSC1_RANGE)
#define CAPSENSE_SNSCLK_SSC2_THRESHOLD                      (CAPSENSE_LFSR_DITHER_PERCENTAGE * CAPSENSE_SNSCLK_SSC2_RANGE)
#define CAPSENSE_SNSCLK_SSC3_THRESHOLD                      (CAPSENSE_LFSR_DITHER_PERCENTAGE * CAPSENSE_SNSCLK_SSC3_RANGE)
#define CAPSENSE_SNSCLK_SSC4_THRESHOLD                      (CAPSENSE_LFSR_DITHER_PERCENTAGE * CAPSENSE_SNSCLK_SSC4_RANGE)
#define CAPSENSE_HFCLK_SNSCLK_FACTOR                        (160uL)

/* Clock register masks */
#define CAPSENSE_MODCLK_CMD_DIV_SHIFT                       (0uL)
#define CAPSENSE_MODCLK_CMD_PA_DIV_SHIFT                    (8uL)
#define CAPSENSE_MODCLK_CMD_DISABLE_SHIFT                   (30uL)
#define CAPSENSE_MODCLK_CMD_ENABLE_SHIFT                    (31uL)
#define CAPSENSE_MODCLK_CMD_DISABLE_MASK                    ((uint32)(1uL << CAPSENSE_MODCLK_CMD_DISABLE_SHIFT))
#define CAPSENSE_MODCLK_CMD_ENABLE_MASK                     ((uint32)(1uL << CAPSENSE_MODCLK_CMD_ENABLE_SHIFT))

#define CAPSENSE_HSIOM_SEL_MASK                             (GPIO_HSIOM_MASK)
#define CAPSENSE_HSIOM_SEL_GPIO                             (HSIOM_SEL_GPIO)
#define CAPSENSE_HSIOM_SEL_CSD_SENSE                        (HSIOM_SEL_ACT_3)
#define CAPSENSE_HSIOM_SEL_CSD_SHIELD                       (HSIOM_SEL_ACT_2)
#define CAPSENSE_HSIOM_SEL_AMUXA                            (HSIOM_SEL_AMUXA)
#define CAPSENSE_HSIOM_SEL_AMUXB                            (HSIOM_SEL_AMUXB)

/***************************************
* API Constants
***************************************/

#define CAPSENSE_MOD_CSD_CLK_12MHZ                          (12000000uL)
#define CAPSENSE_MOD_CSD_CLK_24MHZ                          (24000000uL)
#define CAPSENSE_MOD_CSD_CLK_48MHZ                          (48000000uL)

#define CAPSENSE_MIN_SNS_CLK_DIVIDER                        (4u)

/* Multi-scanning types and constants */
#define CAPSENSE_FREQ_CHANNEL_0                             (0u)
#define CAPSENSE_FREQ_CHANNEL_1                             (1u)
#define CAPSENSE_FREQ_CHANNEL_2                             (2u)

/* Scanning status bit-mask */
#define CAPSENSE_SW_STS_BUSY                                (CAPSENSE_STATUS_CSD0_MASK)
#define CAPSENSE_NOT_BUSY                                   (0uL)

#define CAPSENSE_WDGT_SW_STS_BUSY                           (CAPSENSE_STATUS_WDGT0_BUSY_MASK)

#define CAPSENSE_PERCENTAGE_100                             (0x00000064uL)
#define CAPSENSE_DISCONNECT_IO_FLAG                         (1u)

/* CMOD and CSH capacitor port-pins registers */
#if (CAPSENSE_ENABLE == CAPSENSE_CSD_EN)

    # if defined(CAPSENSE_CSD__DEDICATED_IO1)
        #define CAPSENSE_CTANK_CONNECTION                   (CAPSENSE_CSD__DEDICATED_IO1)
    #else
        #define CAPSENSE_CTANK_CONNECTION                   (0x00000000uL)
    #endif /* CAPSENSE_CSD__DEDICATED_IO1 */

    #if(0u != CAPSENSE_CSX_EN)
        #define CAPSENSE_CMOD_CONNECTION                    (CAPSENSE_CSD__CSHIELD_PAD)
    #else
        #define CAPSENSE_CMOD_CONNECTION                    (CAPSENSE_CSD__DEDICATED_IO0)
    #endif /* (0u != CAPSENSE_CSX_EN) */

    #define CAPSENSE_CSD_CMOD_PORT_PTR                      (CAPSENSE_Cmod_0_PORT)
    #define CAPSENSE_CSD_CMOD_PIN                           (CAPSENSE_Cmod_0_NUM)

    #define CAPSENSE_CSD_CTANK_PORT_PTR                     (CAPSENSE_Csh_0_PORT)
    #define CAPSENSE_CSD_CTANK_PIN                          (CAPSENSE_Csh_0_NUM)

#endif  /* (CAPSENSE_ENABLE == CAPSENSE_CSD_EN) */

#if(CAPSENSE_IREF_SRSS != CAPSENSE_CSD_IREF_SOURCE)
    #define CAPSENSE_IREF_SRC_CFG   (CSD_CONFIG_IREF_SEL_Msk)
#else
    #define CAPSENSE_IREF_SRC_CFG   (0u)
#endif /* (CAPSENSE_CSD_IREF_SRSS != CAPSENSE_CSD_IREF_SOURCE) */

#if(CAPSENSE_ENABLE != CAPSENSE_LP_MODE_EN)
    #define CAPSENSE_PWR_MODE_CFG   (0u)
#else
    #define CAPSENSE_PWR_MODE_CFG   (CSD_CONFIG_LP_MODE_Msk)
#endif /* (CAPSENSE_ENABLE != CAPSENSE_LP_MODE_EN) */

#endif /* End CY_SENSE_CAPSENSE_SENSING_H */


/* [] END OF FILE */
