//===========================================================
// PSoC63 BLE Project
//-----------------------------------------------------------
// File Name   : main_cm4.c
// Description : Main Program for Cortex-M4F
//-----------------------------------------------------------
// History :
// Rev.01 2018.05.20 M.Maruyama First Release
//-----------------------------------------------------------
// Copyright (C) 2018 M.Maruyama
//===========================================================

#include "BLE.h"
#include "CAPSENSE.h"
#include "display.h"
#include "project.h"

//--------------------------
// Globals for BLE Control
//--------------------------
cy_stc_ble_conn_handle_t static connectionHandle; // BLE connection parameters

//-----------------------------------------
// Definitions for LED controlled by BLE 
//-----------------------------------------
#define LED_INDEX  (CY_BLE_LED_LED_CONTROL_CHAR_INDEX)
uint8_t static g_LED_Data = 4; // LED Value

//------------------------------------------------------
// Definitions for SLIDER (CAPSENSE) controlled by BLE
//------------------------------------------------------
#define SLIDER_NOTIFICATION_INDEX    \
(CY_BLE_SLIDER_SLIDER_CONTROL_CLIENT_\
CHARACTERISTIC_CONFIGURATION_DESC_INDEX)
uint8_t static g_Slider_Position = 0xff;  // SLIDER Position (Not Touched as Default Value)
uint8_t static g_Slider_Notify   = 0;     // SLIDER Notification Enable

//--------------
// LED Control 
//--------------
// RED
void Set_LED_RED(uint8_t data)
{
    Cy_GPIO_Write(LED_RED_0_PORT, LED_RED_0_NUM, !data);
}
// ORANGE
void Set_LED_ORG(uint8_t data)
{
    Cy_GPIO_Write(LED_ORG_0_PORT, LED_ORG_0_NUM, !data);
}
// COLOR
void Set_LED_COLOR(uint8_t data)
{
    Cy_GPIO_Write(LED_COLOR_R_0_PORT, LED_COLOR_R_0_NUM, !((data >> 0) & 01));
    Cy_GPIO_Write(LED_COLOR_G_0_PORT, LED_COLOR_G_0_NUM, !((data >> 1) & 01));
    Cy_GPIO_Write(LED_COLOR_B_0_PORT, LED_COLOR_B_0_NUM, !((data >> 2) & 01));
}

//----------------------------
// Get CAPSENSE Slider Data
//----------------------------
void Get_Slider_Data(void)
{
    // When the CAPSENSE isn't Busy
    if (CAPSENSE_IsBusy() == CAPSENSE_NOT_BUSY)
    {
        // Process data from CAPSENSE Widget
		CAPSENSE_ProcessAllWidgets();
        
        // Read the finger position on the slider
        g_Slider_Position = (uint8_t)CAPSENSE_GetCentroidPos(CAPSENSE_LINEARSLIDER_WDGT_ID);
                                               
        // Start next CAPSENSE Scan
        CAPSENSE_ScanAllWidgets();
    }
}

//------------------
// Update LED
//------------------
void Update_LED(void)
{
    // Local variable that stores LED control data parameters
    cy_stc_ble_gatt_handle_value_pair_t  ledHandle = 
    {
        .attrHandle = CY_BLE_LED_LED_CONTROL_CHAR_HANDLE,
        .value.val  = &g_LED_Data,
        .value.len  = 1 // byte 
    };
    
    // Local variable that stores LED attribute value
    cy_stc_ble_gatts_db_attr_val_info_t  ledAttributeValue = 
    {
        .handleValuePair = ledHandle,
        .offset = CY_BLE_CCCD_DEFAULT,
        .connHandle = connectionHandle,
        .flags = CY_BLE_GATT_DB_LOCALLY_INITIATED
    };

    // Send updated LED control handle as an attribute to the central device, 
    // so that the central reads the new LED data
    Cy_BLE_GATTS_WriteAttributeValue(&ledAttributeValue);
}

//-----------------------------
// Update SLIDER Notification
//-----------------------------
void Update_SLIDER_Notification(void)
{
    // Local variable to store the current CCCD value
    uint8_t cccdValue[CY_BLE_CCCD_LEN];
    
    // Load the notification status to the CCCD array
    cccdValue[0] = g_Slider_Notify;
    cccdValue[1] = CY_BLE_CCCD_DEFAULT;
        
    // Local variable that stores notification data parameters
    cy_stc_ble_gatt_handle_value_pair_t  cccdValuePair = 
    {
        .attrHandle = CY_BLE_SLIDER_SLIDER_CONTROL_CLIENT_CHARACTERISTIC_CONFIGURATION_DESC_HANDLE,
        .value.len = CY_BLE_CCCD_LEN,
        .value.val = cccdValue
    };
    
    // Local variable that stores attribute value
    cy_stc_ble_gatts_db_attr_val_info_t  cccdAttributeHandle=
    {
        .connHandle = connectionHandle,
        .handleValuePair = cccdValuePair,
        .offset = CY_BLE_CCCD_DEFAULT,
    };
    
    // Extract flag value from the connection handle
    cccdAttributeHandle.flags = (connectionHandle.bdHandle == 0)?
        CY_BLE_GATT_DB_LOCALLY_INITIATED : CY_BLE_GATT_DB_PEER_INITIATED;
        
    // Update the CCCD attribute value per the input parameters
    Cy_BLE_GATTS_WriteAttributeValueCCCD(&cccdAttributeHandle);
}

//----------------------------------------------------
// Callback Event Function
// to handle various events from the BLE stack
//----------------------------------------------------
void Stack_Event_Handler(uint32_t event, void *eventParameter)
{
    // Local variable to store the data
    // received as part of the write request events
    cy_stc_ble_gatts_write_cmd_req_param_t   *writeReqParameter;
    
    // Take an action based on the current event
    switch (event)
    {

        // Disconnect Device or Stack just Turned on from Reset
        case CY_BLE_EVT_STACK_ON:
        case CY_BLE_EVT_GATT_DISCONNECT_IND:
        {
            // Start the Advertising
            Cy_BLE_GAPP_StartAdvertisement
                (CY_BLE_ADVERTISING_FAST,CY_BLE_PERIPHERAL_CONFIGURATION_0_INDEX);	
            // Set Advertising Indication LED
            Set_LED_RED(1);
            break;
        }
                
        // Device Connected over GATT level
        case CY_BLE_EVT_GATT_CONNECT_IND:
        {
            // Update attribute handle on GATT Connection
            connectionHandle = *(cy_stc_ble_conn_handle_t *) eventParameter;
            // Clear Advertising Indication LED
            Set_LED_RED(0);
            // Update BLE Data to Initial Values
            Update_LED();
            Update_SLIDER_Notification();
            
            break;
        }
        
        // Write Command from Central Device
        case CY_BLE_EVT_GATTS_WRITE_REQ:
        {
            // Read the write request parameter
            writeReqParameter = (cy_stc_ble_gatts_write_cmd_req_param_t *) eventParameter;
			      
            // LED Write Request
            if(writeReqParameter->handleValPair.attrHandle == CY_BLE_LED_LED_CONTROL_CHAR_HANDLE)
            {
                g_LED_Data = writeReqParameter->handleValPair.value.val[LED_INDEX];
                Update_LED();
            }
            
            // CAPSENSE Write Request
            if(writeReqParameter->handleValPair.attrHandle == CY_BLE_SLIDER_SLIDER_CONTROL_CLIENT_CHARACTERISTIC_CONFIGURATION_DESC_HANDLE)
            {
                g_Slider_Notify = writeReqParameter->handleValPair.value.val[SLIDER_NOTIFICATION_INDEX];
                Update_SLIDER_Notification();
            }
            
            // Send the response to the write request received.
            Cy_BLE_GATTS_WriteRsp(connectionHandle);
            break;
        }
            
        // Do nothing for all other events
        default: break;
    }    
}

//----------------------------------------------------
// Function that continuously process the BLE events
// and handles custom BLE services
//----------------------------------------------------
void Process_BLE_Events(void)
{
    uint8_t static slider_Position_prev = 0xff; // Not Untouched as Default
    
    // Process Event Callback to handle BLE events. 
    Cy_BLE_ProcessEvents();
    //
    // Update Physical LED on Board
    Set_LED_COLOR(g_LED_Data); // Color RGB LED
    //
    // BLE is connected ?
	if(Cy_BLE_GetConnectionState(connectionHandle) == CY_BLE_CONN_STATE_CONNECTED)
	{
        // Check if BLE CapSense service notifications is enabled
        if (g_Slider_Notify)
        {
            Get_Slider_Data();
            //
            if (g_Slider_Position != slider_Position_prev)
            {
                if (Cy_BLE_GATT_GetBusyStatus(connectionHandle.attId) == CY_BLE_STACK_STATE_FREE)
                {
                    // Local variable that stores CapSense notification data parameters
                    cy_stc_ble_gatts_handle_value_ntf_t sliderNotificationHandle =
                    {
                        .connHandle = connectionHandle,
                        .handleValPair.attrHandle = 
                                CY_BLE_SLIDER_SLIDER_CONTROL_CHAR_HANDLE,
                        .handleValPair.value.val = &(g_Slider_Position),
                        .handleValPair.value.len = 1 // byte
                    };
            
                    // Send the updated handle as part of attribute for notifications
                    Cy_BLE_GATTS_Notification(&sliderNotificationHandle);
                    
                    // Update Previous Position
                    slider_Position_prev = g_Slider_Position;
                }
            }            
        }
	}
}

//----------------------------------------
// Main Routine
//----------------------------------------
int main(void)
{
    // Enable Global Interrupts.
    __enable_irq();

    // Initialization
    Init_Display();
    CAPSENSE_Start();
    Cy_BLE_Start(Stack_Event_Handler);
    
    // Display Welcome Message
    Display_Welcome();
    
    // Forever Loop
    for(;;)
    {
        // Indicate Active Power Mode
        Set_LED_ORG(1);
        
        // Process BLE Events
        Process_BLE_Events();
        
        // Low Power Control
        // In "Dual-core architecture" for BLE, 
        // then CM0+ (controller) wakes-up CM4 (Host)
        // through the IPC SystemPipe, whenever
        // there is a need to process data on the host side.
        //
        // BLE will automatically wake up the system if an event occurs.
        // In the dual core mode, CM0+ (BLE controller) generates an interrupt
        // to wake up the CM4 if further processing is required. 
        bool gotoLPM = true;
        gotoLPM = (CAPSENSE_IsBusy() != CAPSENSE_NOT_BUSY)? false :
                  (Cy_BLE_GetState() != CY_BLE_STATE_ON)? false :
                  (Cy_BLE_GetNumOfActiveConn() != 0u)? false : gotoLPM;
        if (gotoLPM)
        {
            // Indicate Deep-Sleep Power Mode
            Set_LED_ORG(0);
            // Goto Deep-Sleep Mode
            Cy_SysPm_DeepSleep(CY_SYSPM_WAIT_FOR_INTERRUPT);
        }
    }
}

//===========================================================
// End of Program
//===========================================================
